#
# Generated for Onboard by gen_unicode_data
#

emoji_data = [
    [0, '🙂',                                      # category: face-positive
        [
            ('😀', None),                          # 😀 grinning face
            ('😁', None),                          # 😁 grinning face with smiling eyes
            ('😂', None),                          # 😂 face with tears of joy
            ('\U0001f923', None),                 # 🤣 rolling on the floor laughing
            ('😃', None),                          # 😃 smiling face with open mouth
            ('😄', None),                          # 😄 smiling face with open mouth & smiling eyes
            ('😅', None),                          # 😅 smiling face with open mouth & cold sweat
            ('😆', None),                          # 😆 smiling face with open mouth & closed eyes
            ('😉', None),                          # 😉 winking face
            ('😊', None),                          # 😊 smiling face with smiling eyes
            ('😋', None),                          # 😋 face savouring delicious food
            ('😎', None),                          # 😎 smiling face with sunglasses
            ('😍', None),                          # 😍 smiling face with heart-eyes
            ('😘', None),                          # 😘 face blowing a kiss
            ('😗', None),                          # 😗 kissing face
            ('😙', None),                          # 😙 kissing face with smiling eyes
            ('😚', None),                          # 😚 kissing face with closed eyes
            ('☺', None),                          # ☺ smiling face
            ('🙂', None),                          # 🙂 slightly smiling face
            ('🤗', None),                          # 🤗 hugging face
        ]],
    [1, '😐',                                      # category: face-neutral
        [
            ('🤔', None),                          # 🤔 thinking face
            ('😐', None),                          # 😐 neutral face
            ('😑', None),                          # 😑 expressionless face
            ('😶', None),                          # 😶 face without mouth
            ('🙄', None),                          # 🙄 face with rolling eyes
            ('😏', None),                          # 😏 smirking face
            ('😣', None),                          # 😣 persevering face
            ('😥', None),                          # 😥 disappointed but relieved face
            ('😮', None),                          # 😮 face with open mouth
            ('🤐', None),                          # 🤐 zipper-mouth face
            ('😯', None),                          # 😯 hushed face
            ('😪', None),                          # 😪 sleepy face
            ('😫', None),                          # 😫 tired face
            ('😴', None),                          # 😴 sleeping face
            ('😌', None),                          # 😌 relieved face
            ('🤓', None),                          # 🤓 nerd face
            ('😛', None),                          # 😛 face with stuck-out tongue
            ('😜', None),                          # 😜 face with stuck-out tongue & winking eye
            ('😝', None),                          # 😝 face with stuck-out tongue & closed eyes
            ('\U0001f924', None),                 # 🤤 drooling face
            ('😒', None),                          # 😒 unamused face
            ('😓', None),                          # 😓 face with cold sweat
            ('😔', None),                          # 😔 pensive face
            ('😕', None),                          # 😕 confused face
            ('🙃', None),                          # 🙃 upside-down face
            ('🤑', None),                          # 🤑 money-mouth face
            ('😲', None),                          # 😲 astonished face
        ]],
    [1, '☹',                                      # category: face-negative
        [
            ('☹', None),                          # ☹ frowning face
            ('🙁', None),                          # 🙁 slightly frowning face
            ('😖', None),                          # 😖 confounded face
            ('😞', None),                          # 😞 disappointed face
            ('😟', None),                          # 😟 worried face
            ('😤', None),                          # 😤 face with steam from nose
            ('😢', None),                          # 😢 crying face
            ('😭', None),                          # 😭 loudly crying face
            ('😦', None),                          # 😦 frowning face with open mouth
            ('😧', None),                          # 😧 anguished face
            ('😨', None),                          # 😨 fearful face
            ('😩', None),                          # 😩 weary face
            ('😬', None),                          # 😬 grimacing face
            ('😰', None),                          # 😰 face with open mouth & cold sweat
            ('😱', None),                          # 😱 face screaming in fear
            ('😳', None),                          # 😳 flushed face
            ('😵', None),                          # 😵 dizzy face
            ('😡', None),                          # 😡 pouting face
            ('😠', None),                          # 😠 angry face
        ]],
    [1, '😇',                                      # category: face-role
        [
            ('😇', None),                          # 😇 smiling face with halo
            ('\U0001f920', None),                 # 🤠 cowboy hat face
            ('\U0001f921', None),                 # 🤡 clown face
            ('\U0001f925', None),                 # 🤥 lying face
        ]],
    [1, '😷',                                      # category: face-sick
        [
            ('😷', None),                          # 😷 face with medical mask
            ('🤒', None),                          # 🤒 face with thermometer
            ('🤕', None),                          # 🤕 face with head-bandage
            ('\U0001f922', None),                 # 🤢 nauseated face
            ('\U0001f927', None),                 # 🤧 sneezing face
        ]],
    [1, '😈',                                      # category: creature-face
        [
            ('😈', None),                          # 😈 smiling face with horns
            ('👿', None),                          # 👿 angry face with horns
            ('👹', None),                          # 👹 ogre
            ('👺', None),                          # 👺 goblin
            ('💀', None),                          # 💀 skull
            ('☠', None),                          # ☠ skull and crossbones
            ('👻', None),                          # 👻 ghost
            ('👽', None),                          # 👽 alien
            ('👾', None),                          # 👾 alien monster
            ('🤖', None),                          # 🤖 robot face
            ('💩', None),                          # 💩 pile of poo
        ]],
    [1, '😺',                                      # category: cat-face
        [
            ('😺', None),                          # 😺 smiling cat face with open mouth
            ('😸', None),                          # 😸 grinning cat face with smiling eyes
            ('😹', None),                          # 😹 cat face with tears of joy
            ('😻', None),                          # 😻 smiling cat face with heart-eyes
            ('😼', None),                          # 😼 cat face with wry smile
            ('😽', None),                          # 😽 kissing cat face with closed eyes
            ('🙀', None),                          # 🙀 weary cat face
            ('😿', None),                          # 😿 crying cat face
            ('😾', None),                          # 😾 pouting cat face
        ]],
    [1, '🙈',                                      # category: monkey-face
        [
            ('🙈', None),                          # 🙈 see-no-evil monkey
            ('🙉', None),                          # 🙉 hear-no-evil monkey
            ('🙊', None),                          # 🙊 speak-no-evil monkey
        ]],
    [1, '👦',                                      # category: person
        [
            ('👦',                                 # 👦 boy
                ('👦🏻',                            # 👦🏻 boy: light skin tone
                 '👦🏼',                            # 👦🏼 boy: medium-light skin tone
                 '👦🏽',                            # 👦🏽 boy: medium skin tone
                 '👦🏾',                            # 👦🏾 boy: medium-dark skin tone
                 '👦🏿')),                          # 👦🏿 boy: dark skin tone
            ('👧',                                 # 👧 girl
                ('👧🏻',                            # 👧🏻 girl: light skin tone
                 '👧🏼',                            # 👧🏼 girl: medium-light skin tone
                 '👧🏽',                            # 👧🏽 girl: medium skin tone
                 '👧🏾',                            # 👧🏾 girl: medium-dark skin tone
                 '👧🏿')),                          # 👧🏿 girl: dark skin tone
            ('👨',                                 # 👨 man
                ('👨🏻',                            # 👨🏻 man: light skin tone
                 '👨🏼',                            # 👨🏼 man: medium-light skin tone
                 '👨🏽',                            # 👨🏽 man: medium skin tone
                 '👨🏾',                            # 👨🏾 man: medium-dark skin tone
                 '👨🏿')),                          # 👨🏿 man: dark skin tone
            ('👩',                                 # 👩 woman
                ('👩🏻',                            # 👩🏻 woman: light skin tone
                 '👩🏼',                            # 👩🏼 woman: medium-light skin tone
                 '👩🏽',                            # 👩🏽 woman: medium skin tone
                 '👩🏾',                            # 👩🏾 woman: medium-dark skin tone
                 '👩🏿')),                          # 👩🏿 woman: dark skin tone
            ('👴',                                 # 👴 old man
                ('👴🏻',                            # 👴🏻 old man: light skin tone
                 '👴🏼',                            # 👴🏼 old man: medium-light skin tone
                 '👴🏽',                            # 👴🏽 old man: medium skin tone
                 '👴🏾',                            # 👴🏾 old man: medium-dark skin tone
                 '👴🏿')),                          # 👴🏿 old man: dark skin tone
            ('👵',                                 # 👵 old woman
                ('👵🏻',                            # 👵🏻 old woman: light skin tone
                 '👵🏼',                            # 👵🏼 old woman: medium-light skin tone
                 '👵🏽',                            # 👵🏽 old woman: medium skin tone
                 '👵🏾',                            # 👵🏾 old woman: medium-dark skin tone
                 '👵🏿')),                          # 👵🏿 old woman: dark skin tone
            ('👶',                                 # 👶 baby
                ('👶🏻',                            # 👶🏻 baby: light skin tone
                 '👶🏼',                            # 👶🏼 baby: medium-light skin tone
                 '👶🏽',                            # 👶🏽 baby: medium skin tone
                 '👶🏾',                            # 👶🏾 baby: medium-dark skin tone
                 '👶🏿')),                          # 👶🏿 baby: dark skin tone
            ('👼',                                 # 👼 baby angel
                ('👼🏻',                            # 👼🏻 baby angel: light skin tone
                 '👼🏼',                            # 👼🏼 baby angel: medium-light skin tone
                 '👼🏽',                            # 👼🏽 baby angel: medium skin tone
                 '👼🏾',                            # 👼🏾 baby angel: medium-dark skin tone
                 '👼🏿')),                          # 👼🏿 baby angel: dark skin tone
        ]],
    [1, '👮',                                      # category: person-role
        [
            ('👮',                                 # 👮 police officer
                ('👮🏻',                            # 👮🏻 police officer: light skin tone
                 '👮🏼',                            # 👮🏼 police officer: medium-light skin tone
                 '👮🏽',                            # 👮🏽 police officer: medium skin tone
                 '👮🏾',                            # 👮🏾 police officer: medium-dark skin tone
                 '👮🏿')),                          # 👮🏿 police officer: dark skin tone
            ('🕵',                                 # 🕵 detective
                ('🕵🏻',                            # 🕵🏻 detective: light skin tone
                 '🕵🏼',                            # 🕵🏼 detective: medium-light skin tone
                 '🕵🏽',                            # 🕵🏽 detective: medium skin tone
                 '🕵🏾',                            # 🕵🏾 detective: medium-dark skin tone
                 '🕵🏿')),                          # 🕵🏿 detective: dark skin tone
            ('💂',                                 # 💂 guard
                ('💂🏻',                            # 💂🏻 guard: light skin tone
                 '💂🏼',                            # 💂🏼 guard: medium-light skin tone
                 '💂🏽',                            # 💂🏽 guard: medium skin tone
                 '💂🏾',                            # 💂🏾 guard: medium-dark skin tone
                 '💂🏿')),                          # 💂🏿 guard: dark skin tone
            ('👷',                                 # 👷 construction worker
                ('👷🏻',                            # 👷🏻 construction worker: light skin tone
                 '👷🏼',                            # 👷🏼 construction worker: medium-light skin tone
                 '👷🏽',                            # 👷🏽 construction worker: medium skin tone
                 '👷🏾',                            # 👷🏾 construction worker: medium-dark skin tone
                 '👷🏿')),                          # 👷🏿 construction worker: dark skin tone
            ('👳',                                 # 👳 person wearing turban
                ('👳🏻',                            # 👳🏻 person wearing turban: light skin tone
                 '👳🏼',                            # 👳🏼 person wearing turban: medium-light skin tone
                 '👳🏽',                            # 👳🏽 person wearing turban: medium skin tone
                 '👳🏾',                            # 👳🏾 person wearing turban: medium-dark skin tone
                 '👳🏿')),                          # 👳🏿 person wearing turban: dark skin tone
            ('👱',                                 # 👱 blond-haired person
                ('👱🏻',                            # 👱🏻 blond-haired person: light skin tone
                 '👱🏼',                            # 👱🏼 blond-haired person: medium-light skin tone
                 '👱🏽',                            # 👱🏽 blond-haired person: medium skin tone
                 '👱🏾',                            # 👱🏾 blond-haired person: medium-dark skin tone
                 '👱🏿')),                          # 👱🏿 blond-haired person: dark skin tone
            ('🎅',                                 # 🎅 Santa Claus
                ('🎅🏻',                            # 🎅🏻 Santa Claus: light skin tone
                 '🎅🏼',                            # 🎅🏼 Santa Claus: medium-light skin tone
                 '🎅🏽',                            # 🎅🏽 Santa Claus: medium skin tone
                 '🎅🏾',                            # 🎅🏾 Santa Claus: medium-dark skin tone
                 '🎅🏿')),                          # 🎅🏿 Santa Claus: dark skin tone
            ('\U0001f936',                        # 🤶 Mrs. Claus
                ('\U0001f936🏻',                   # 🤶🏻 Mrs. Claus: light skin tone
                 '\U0001f936🏼',                   # 🤶🏼 Mrs. Claus: medium-light skin tone
                 '\U0001f936🏽',                   # 🤶🏽 Mrs. Claus: medium skin tone
                 '\U0001f936🏾',                   # 🤶🏾 Mrs. Claus: medium-dark skin tone
                 '\U0001f936🏿')),                 # 🤶🏿 Mrs. Claus: dark skin tone
            ('👸',                                 # 👸 princess
                ('👸🏻',                            # 👸🏻 princess: light skin tone
                 '👸🏼',                            # 👸🏼 princess: medium-light skin tone
                 '👸🏽',                            # 👸🏽 princess: medium skin tone
                 '👸🏾',                            # 👸🏾 princess: medium-dark skin tone
                 '👸🏿')),                          # 👸🏿 princess: dark skin tone
            ('\U0001f934',                        # 🤴 prince
                ('\U0001f934🏻',                   # 🤴🏻 prince: light skin tone
                 '\U0001f934🏼',                   # 🤴🏼 prince: medium-light skin tone
                 '\U0001f934🏽',                   # 🤴🏽 prince: medium skin tone
                 '\U0001f934🏾',                   # 🤴🏾 prince: medium-dark skin tone
                 '\U0001f934🏿')),                 # 🤴🏿 prince: dark skin tone
            ('👰',                                 # 👰 bride with veil
                ('👰🏻',                            # 👰🏻 bride with veil: light skin tone
                 '👰🏼',                            # 👰🏼 bride with veil: medium-light skin tone
                 '👰🏽',                            # 👰🏽 bride with veil: medium skin tone
                 '👰🏾',                            # 👰🏾 bride with veil: medium-dark skin tone
                 '👰🏿')),                          # 👰🏿 bride with veil: dark skin tone
            ('\U0001f935',                        # 🤵 man in tuxedo
                ('\U0001f935🏻',                   # 🤵🏻 man in tuxedo: light skin tone
                 '\U0001f935🏼',                   # 🤵🏼 man in tuxedo: medium-light skin tone
                 '\U0001f935🏽',                   # 🤵🏽 man in tuxedo: medium skin tone
                 '\U0001f935🏾',                   # 🤵🏾 man in tuxedo: medium-dark skin tone
                 '\U0001f935🏿')),                 # 🤵🏿 man in tuxedo: dark skin tone
            ('\U0001f930',                        # 🤰 pregnant woman
                ('\U0001f930🏻',                   # 🤰🏻 pregnant woman: light skin tone
                 '\U0001f930🏼',                   # 🤰🏼 pregnant woman: medium-light skin tone
                 '\U0001f930🏽',                   # 🤰🏽 pregnant woman: medium skin tone
                 '\U0001f930🏾',                   # 🤰🏾 pregnant woman: medium-dark skin tone
                 '\U0001f930🏿')),                 # 🤰🏿 pregnant woman: dark skin tone
            ('👲',                                 # 👲 man with Chinese cap
                ('👲🏻',                            # 👲🏻 man with Chinese cap: light skin tone
                 '👲🏼',                            # 👲🏼 man with Chinese cap: medium-light skin tone
                 '👲🏽',                            # 👲🏽 man with Chinese cap: medium skin tone
                 '👲🏾',                            # 👲🏾 man with Chinese cap: medium-dark skin tone
                 '👲🏿')),                          # 👲🏿 man with Chinese cap: dark skin tone
        ]],
    [1, '🙍',                                      # category: person-gesture
        [
            ('🙍',                                 # 🙍 person frowning
                ('🙍🏻',                            # 🙍🏻 person frowning: light skin tone
                 '🙍🏼',                            # 🙍🏼 person frowning: medium-light skin tone
                 '🙍🏽',                            # 🙍🏽 person frowning: medium skin tone
                 '🙍🏾',                            # 🙍🏾 person frowning: medium-dark skin tone
                 '🙍🏿')),                          # 🙍🏿 person frowning: dark skin tone
            ('🙎',                                 # 🙎 person pouting
                ('🙎🏻',                            # 🙎🏻 person pouting: light skin tone
                 '🙎🏼',                            # 🙎🏼 person pouting: medium-light skin tone
                 '🙎🏽',                            # 🙎🏽 person pouting: medium skin tone
                 '🙎🏾',                            # 🙎🏾 person pouting: medium-dark skin tone
                 '🙎🏿')),                          # 🙎🏿 person pouting: dark skin tone
            ('🙅',                                 # 🙅 person gesturing NO
                ('🙅🏻',                            # 🙅🏻 person gesturing NO: light skin tone
                 '🙅🏼',                            # 🙅🏼 person gesturing NO: medium-light skin tone
                 '🙅🏽',                            # 🙅🏽 person gesturing NO: medium skin tone
                 '🙅🏾',                            # 🙅🏾 person gesturing NO: medium-dark skin tone
                 '🙅🏿')),                          # 🙅🏿 person gesturing NO: dark skin tone
            ('🙆',                                 # 🙆 person gesturing OK
                ('🙆🏻',                            # 🙆🏻 person gesturing OK: light skin tone
                 '🙆🏼',                            # 🙆🏼 person gesturing OK: medium-light skin tone
                 '🙆🏽',                            # 🙆🏽 person gesturing OK: medium skin tone
                 '🙆🏾',                            # 🙆🏾 person gesturing OK: medium-dark skin tone
                 '🙆🏿')),                          # 🙆🏿 person gesturing OK: dark skin tone
            ('💁',                                 # 💁 person tipping hand
                ('💁🏻',                            # 💁🏻 person tipping hand: light skin tone
                 '💁🏼',                            # 💁🏼 person tipping hand: medium-light skin tone
                 '💁🏽',                            # 💁🏽 person tipping hand: medium skin tone
                 '💁🏾',                            # 💁🏾 person tipping hand: medium-dark skin tone
                 '💁🏿')),                          # 💁🏿 person tipping hand: dark skin tone
            ('🙋',                                 # 🙋 person raising hand
                ('🙋🏻',                            # 🙋🏻 person raising hand: light skin tone
                 '🙋🏼',                            # 🙋🏼 person raising hand: medium-light skin tone
                 '🙋🏽',                            # 🙋🏽 person raising hand: medium skin tone
                 '🙋🏾',                            # 🙋🏾 person raising hand: medium-dark skin tone
                 '🙋🏿')),                          # 🙋🏿 person raising hand: dark skin tone
            ('🙇',                                 # 🙇 person bowing
                ('🙇🏻',                            # 🙇🏻 person bowing: light skin tone
                 '🙇🏼',                            # 🙇🏼 person bowing: medium-light skin tone
                 '🙇🏽',                            # 🙇🏽 person bowing: medium skin tone
                 '🙇🏾',                            # 🙇🏾 person bowing: medium-dark skin tone
                 '🙇🏿')),                          # 🙇🏿 person bowing: dark skin tone
            ('\U0001f926',                        # 🤦 person facepalming
                ('\U0001f926🏻',                   # 🤦🏻 person facepalming: light skin tone
                 '\U0001f926🏼',                   # 🤦🏼 person facepalming: medium-light skin tone
                 '\U0001f926🏽',                   # 🤦🏽 person facepalming: medium skin tone
                 '\U0001f926🏾',                   # 🤦🏾 person facepalming: medium-dark skin tone
                 '\U0001f926🏿')),                 # 🤦🏿 person facepalming: dark skin tone
            ('\U0001f937',                        # 🤷 person shrugging
                ('\U0001f937🏻',                   # 🤷🏻 person shrugging: light skin tone
                 '\U0001f937🏼',                   # 🤷🏼 person shrugging: medium-light skin tone
                 '\U0001f937🏽',                   # 🤷🏽 person shrugging: medium skin tone
                 '\U0001f937🏾',                   # 🤷🏾 person shrugging: medium-dark skin tone
                 '\U0001f937🏿')),                 # 🤷🏿 person shrugging: dark skin tone
        ]],
    [1, '💆',                                      # category: person-activity
        [
            ('💆',                                 # 💆 person getting massage
                ('💆🏻',                            # 💆🏻 person getting massage: light skin tone
                 '💆🏼',                            # 💆🏼 person getting massage: medium-light skin tone
                 '💆🏽',                            # 💆🏽 person getting massage: medium skin tone
                 '💆🏾',                            # 💆🏾 person getting massage: medium-dark skin tone
                 '💆🏿')),                          # 💆🏿 person getting massage: dark skin tone
            ('💇',                                 # 💇 person getting haircut
                ('💇🏻',                            # 💇🏻 person getting haircut: light skin tone
                 '💇🏼',                            # 💇🏼 person getting haircut: medium-light skin tone
                 '💇🏽',                            # 💇🏽 person getting haircut: medium skin tone
                 '💇🏾',                            # 💇🏾 person getting haircut: medium-dark skin tone
                 '💇🏿')),                          # 💇🏿 person getting haircut: dark skin tone
            ('🚶',                                 # 🚶 person walking
                ('🚶🏻',                            # 🚶🏻 person walking: light skin tone
                 '🚶🏼',                            # 🚶🏼 person walking: medium-light skin tone
                 '🚶🏽',                            # 🚶🏽 person walking: medium skin tone
                 '🚶🏾',                            # 🚶🏾 person walking: medium-dark skin tone
                 '🚶🏿')),                          # 🚶🏿 person walking: dark skin tone
            ('🏃',                                 # 🏃 person running
                ('🏃🏻',                            # 🏃🏻 person running: light skin tone
                 '🏃🏼',                            # 🏃🏼 person running: medium-light skin tone
                 '🏃🏽',                            # 🏃🏽 person running: medium skin tone
                 '🏃🏾',                            # 🏃🏾 person running: medium-dark skin tone
                 '🏃🏿')),                          # 🏃🏿 person running: dark skin tone
            ('💃',                                 # 💃 woman dancing
                ('💃🏻',                            # 💃🏻 woman dancing: light skin tone
                 '💃🏼',                            # 💃🏼 woman dancing: medium-light skin tone
                 '💃🏽',                            # 💃🏽 woman dancing: medium skin tone
                 '💃🏾',                            # 💃🏾 woman dancing: medium-dark skin tone
                 '💃🏿')),                          # 💃🏿 woman dancing: dark skin tone
            ('\U0001f57a',                        # 🕺 man dancing
                ('\U0001f57a🏻',                   # 🕺🏻 man dancing: light skin tone
                 '\U0001f57a🏼',                   # 🕺🏼 man dancing: medium-light skin tone
                 '\U0001f57a🏽',                   # 🕺🏽 man dancing: medium skin tone
                 '\U0001f57a🏾',                   # 🕺🏾 man dancing: medium-dark skin tone
                 '\U0001f57a🏿')),                 # 🕺🏿 man dancing: dark skin tone
            ('👯', None),                          # 👯 people with bunny ears partying
            ('🕴',                                 # 🕴 man in business suit levitating
                ('🕴🏻',                            # 🕴🏻 man in business suit levitating: light skin tone
                 '🕴🏼',                            # 🕴🏼 man in business suit levitating: medium-light skin tone
                 '🕴🏽',                            # 🕴🏽 man in business suit levitating: medium skin tone
                 '🕴🏾',                            # 🕴🏾 man in business suit levitating: medium-dark skin tone
                 '🕴🏿')),                          # 🕴🏿 man in business suit levitating: dark skin tone
            ('🗣', None),                          # 🗣 speaking head
            ('👤', None),                          # 👤 bust in silhouette
            ('👥', None),                          # 👥 busts in silhouette
        ]],
    [1, '\U0001f93a',                             # category: person-sport
        [
            ('\U0001f93a', None),                 # 🤺 person fencing
            ('🏇',                                 # 🏇 horse racing
                ('🏇🏻',                            # 🏇🏻 horse racing: light skin tone
                 '🏇🏼',                            # 🏇🏼 horse racing: medium-light skin tone
                 '🏇🏽',                            # 🏇🏽 horse racing: medium skin tone
                 '🏇🏾',                            # 🏇🏾 horse racing: medium-dark skin tone
                 '🏇🏿')),                          # 🏇🏿 horse racing: dark skin tone
            ('⛷', None),                          # ⛷ skier
            ('🏂',                                 # 🏂 snowboarder
                ('🏂🏻',                            # 🏂🏻 snowboarder: light skin tone
                 '🏂🏼',                            # 🏂🏼 snowboarder: medium-light skin tone
                 '🏂🏽',                            # 🏂🏽 snowboarder: medium skin tone
                 '🏂🏾',                            # 🏂🏾 snowboarder: medium-dark skin tone
                 '🏂🏿')),                          # 🏂🏿 snowboarder: dark skin tone
            ('🏌',                                 # 🏌 person golfing
                ('🏌🏻',                            # 🏌🏻 person golfing: light skin tone
                 '🏌🏼',                            # 🏌🏼 person golfing: medium-light skin tone
                 '🏌🏽',                            # 🏌🏽 person golfing: medium skin tone
                 '🏌🏾',                            # 🏌🏾 person golfing: medium-dark skin tone
                 '🏌🏿')),                          # 🏌🏿 person golfing: dark skin tone
            ('🏄',                                 # 🏄 person surfing
                ('🏄🏻',                            # 🏄🏻 person surfing: light skin tone
                 '🏄🏼',                            # 🏄🏼 person surfing: medium-light skin tone
                 '🏄🏽',                            # 🏄🏽 person surfing: medium skin tone
                 '🏄🏾',                            # 🏄🏾 person surfing: medium-dark skin tone
                 '🏄🏿')),                          # 🏄🏿 person surfing: dark skin tone
            ('🚣',                                 # 🚣 person rowing boat
                ('🚣🏻',                            # 🚣🏻 person rowing boat: light skin tone
                 '🚣🏼',                            # 🚣🏼 person rowing boat: medium-light skin tone
                 '🚣🏽',                            # 🚣🏽 person rowing boat: medium skin tone
                 '🚣🏾',                            # 🚣🏾 person rowing boat: medium-dark skin tone
                 '🚣🏿')),                          # 🚣🏿 person rowing boat: dark skin tone
            ('🏊',                                 # 🏊 person swimming
                ('🏊🏻',                            # 🏊🏻 person swimming: light skin tone
                 '🏊🏼',                            # 🏊🏼 person swimming: medium-light skin tone
                 '🏊🏽',                            # 🏊🏽 person swimming: medium skin tone
                 '🏊🏾',                            # 🏊🏾 person swimming: medium-dark skin tone
                 '🏊🏿')),                          # 🏊🏿 person swimming: dark skin tone
            ('⛹',                                 # ⛹ person bouncing ball
                ('⛹🏻',                            # ⛹🏻 person bouncing ball: light skin tone
                 '⛹🏼',                            # ⛹🏼 person bouncing ball: medium-light skin tone
                 '⛹🏽',                            # ⛹🏽 person bouncing ball: medium skin tone
                 '⛹🏾',                            # ⛹🏾 person bouncing ball: medium-dark skin tone
                 '⛹🏿')),                          # ⛹🏿 person bouncing ball: dark skin tone
            ('🏋',                                 # 🏋 person lifting weights
                ('🏋🏻',                            # 🏋🏻 person lifting weights: light skin tone
                 '🏋🏼',                            # 🏋🏼 person lifting weights: medium-light skin tone
                 '🏋🏽',                            # 🏋🏽 person lifting weights: medium skin tone
                 '🏋🏾',                            # 🏋🏾 person lifting weights: medium-dark skin tone
                 '🏋🏿')),                          # 🏋🏿 person lifting weights: dark skin tone
            ('🚴',                                 # 🚴 person biking
                ('🚴🏻',                            # 🚴🏻 person biking: light skin tone
                 '🚴🏼',                            # 🚴🏼 person biking: medium-light skin tone
                 '🚴🏽',                            # 🚴🏽 person biking: medium skin tone
                 '🚴🏾',                            # 🚴🏾 person biking: medium-dark skin tone
                 '🚴🏿')),                          # 🚴🏿 person biking: dark skin tone
            ('🚵',                                 # 🚵 person mountain biking
                ('🚵🏻',                            # 🚵🏻 person mountain biking: light skin tone
                 '🚵🏼',                            # 🚵🏼 person mountain biking: medium-light skin tone
                 '🚵🏽',                            # 🚵🏽 person mountain biking: medium skin tone
                 '🚵🏾',                            # 🚵🏾 person mountain biking: medium-dark skin tone
                 '🚵🏿')),                          # 🚵🏿 person mountain biking: dark skin tone
            ('🏎', None),                          # 🏎 racing car
            ('🏍', None),                          # 🏍 motorcycle
            ('\U0001f938',                        # 🤸 person cartwheeling
                ('\U0001f938🏻',                   # 🤸🏻 person cartwheeling: light skin tone
                 '\U0001f938🏼',                   # 🤸🏼 person cartwheeling: medium-light skin tone
                 '\U0001f938🏽',                   # 🤸🏽 person cartwheeling: medium skin tone
                 '\U0001f938🏾',                   # 🤸🏾 person cartwheeling: medium-dark skin tone
                 '\U0001f938🏿')),                 # 🤸🏿 person cartwheeling: dark skin tone
            ('\U0001f93c', None),                 # 🤼 people wrestling
            ('\U0001f93d',                        # 🤽 person playing water polo
                ('\U0001f93d🏻',                   # 🤽🏻 person playing water polo: light skin tone
                 '\U0001f93d🏼',                   # 🤽🏼 person playing water polo: medium-light skin tone
                 '\U0001f93d🏽',                   # 🤽🏽 person playing water polo: medium skin tone
                 '\U0001f93d🏾',                   # 🤽🏾 person playing water polo: medium-dark skin tone
                 '\U0001f93d🏿')),                 # 🤽🏿 person playing water polo: dark skin tone
            ('\U0001f93e',                        # 🤾 person playing handball
                ('\U0001f93e🏻',                   # 🤾🏻 person playing handball: light skin tone
                 '\U0001f93e🏼',                   # 🤾🏼 person playing handball: medium-light skin tone
                 '\U0001f93e🏽',                   # 🤾🏽 person playing handball: medium skin tone
                 '\U0001f93e🏾',                   # 🤾🏾 person playing handball: medium-dark skin tone
                 '\U0001f93e🏿')),                 # 🤾🏿 person playing handball: dark skin tone
            ('\U0001f939',                        # 🤹 person juggling
                ('\U0001f939🏻',                   # 🤹🏻 person juggling: light skin tone
                 '\U0001f939🏼',                   # 🤹🏼 person juggling: medium-light skin tone
                 '\U0001f939🏽',                   # 🤹🏽 person juggling: medium skin tone
                 '\U0001f939🏾',                   # 🤹🏾 person juggling: medium-dark skin tone
                 '\U0001f939🏿')),                 # 🤹🏿 person juggling: dark skin tone
        ]],
    [1, '👫',                                      # category: family
        [
            ('👫', None),                          # 👫 man and woman holding hands
            ('👬', None),                          # 👬 two men holding hands
            ('👭', None),                          # 👭 two women holding hands
            ('💏', None),                          # 💏 kiss
            ('👨\u200d❤️\u200d💋\u200d👨', None),    # 👨‍❤️‍💋‍👨 kiss: man, man
            ('👩\u200d❤️\u200d💋\u200d👩', None),    # 👩‍❤️‍💋‍👩 kiss: woman, woman
            ('💑', None),                          # 💑 couple with heart
            ('👨\u200d❤️\u200d👨', None),           # 👨‍❤️‍👨 couple with heart: man, man
            ('👩\u200d❤️\u200d👩', None),           # 👩‍❤️‍👩 couple with heart: woman, woman
            ('👪', None),                          # 👪 family
            ('👨\u200d👩\u200d👧', None),            # 👨‍👩‍👧 family: man, woman, girl
            ('👨\u200d👩\u200d👧\u200d👦', None),     # 👨‍👩‍👧‍👦 family: man, woman, girl, boy
            ('👨\u200d👩\u200d👦\u200d👦', None),     # 👨‍👩‍👦‍👦 family: man, woman, boy, boy
            ('👨\u200d👩\u200d👧\u200d👧', None),     # 👨‍👩‍👧‍👧 family: man, woman, girl, girl
            ('👨\u200d👨\u200d👦', None),            # 👨‍👨‍👦 family: man, man, boy
            ('👨\u200d👨\u200d👧', None),            # 👨‍👨‍👧 family: man, man, girl
            ('👨\u200d👨\u200d👧\u200d👦', None),     # 👨‍👨‍👧‍👦 family: man, man, girl, boy
            ('👨\u200d👨\u200d👦\u200d👦', None),     # 👨‍👨‍👦‍👦 family: man, man, boy, boy
            ('👨\u200d👨\u200d👧\u200d👧', None),     # 👨‍👨‍👧‍👧 family: man, man, girl, girl
            ('👩\u200d👩\u200d👦', None),            # 👩‍👩‍👦 family: woman, woman, boy
            ('👩\u200d👩\u200d👧', None),            # 👩‍👩‍👧 family: woman, woman, girl
            ('👩\u200d👩\u200d👧\u200d👦', None),     # 👩‍👩‍👧‍👦 family: woman, woman, girl, boy
            ('👩\u200d👩\u200d👦\u200d👦', None),     # 👩‍👩‍👦‍👦 family: woman, woman, boy, boy
            ('👩\u200d👩\u200d👧\u200d👧', None),     # 👩‍👩‍👧‍👧 family: woman, woman, girl, girl
        ]],
    [1, '💪',                                      # category: body
        [
            ('💪',                                 # 💪 flexed biceps
                ('💪🏻',                            # 💪🏻 flexed biceps: light skin tone
                 '💪🏼',                            # 💪🏼 flexed biceps: medium-light skin tone
                 '💪🏽',                            # 💪🏽 flexed biceps: medium skin tone
                 '💪🏾',                            # 💪🏾 flexed biceps: medium-dark skin tone
                 '💪🏿')),                          # 💪🏿 flexed biceps: dark skin tone
            ('\U0001f933',                        # 🤳 selfie
                ('\U0001f933🏻',                   # 🤳🏻 selfie: light skin tone
                 '\U0001f933🏼',                   # 🤳🏼 selfie: medium-light skin tone
                 '\U0001f933🏽',                   # 🤳🏽 selfie: medium skin tone
                 '\U0001f933🏾',                   # 🤳🏾 selfie: medium-dark skin tone
                 '\U0001f933🏿')),                 # 🤳🏿 selfie: dark skin tone
            ('👈',                                 # 👈 backhand index pointing left
                ('👈🏻',                            # 👈🏻 backhand index pointing left: light skin tone
                 '👈🏼',                            # 👈🏼 backhand index pointing left: medium-light skin tone
                 '👈🏽',                            # 👈🏽 backhand index pointing left: medium skin tone
                 '👈🏾',                            # 👈🏾 backhand index pointing left: medium-dark skin tone
                 '👈🏿')),                          # 👈🏿 backhand index pointing left: dark skin tone
            ('👉',                                 # 👉 backhand index pointing right
                ('👉🏻',                            # 👉🏻 backhand index pointing right: light skin tone
                 '👉🏼',                            # 👉🏼 backhand index pointing right: medium-light skin tone
                 '👉🏽',                            # 👉🏽 backhand index pointing right: medium skin tone
                 '👉🏾',                            # 👉🏾 backhand index pointing right: medium-dark skin tone
                 '👉🏿')),                          # 👉🏿 backhand index pointing right: dark skin tone
            ('☝',                                 # ☝ index pointing up
                ('☝🏻',                            # ☝🏻 index pointing up: light skin tone
                 '☝🏼',                            # ☝🏼 index pointing up: medium-light skin tone
                 '☝🏽',                            # ☝🏽 index pointing up: medium skin tone
                 '☝🏾',                            # ☝🏾 index pointing up: medium-dark skin tone
                 '☝🏿')),                          # ☝🏿 index pointing up: dark skin tone
            ('👆',                                 # 👆 backhand index pointing up
                ('👆🏻',                            # 👆🏻 backhand index pointing up: light skin tone
                 '👆🏼',                            # 👆🏼 backhand index pointing up: medium-light skin tone
                 '👆🏽',                            # 👆🏽 backhand index pointing up: medium skin tone
                 '👆🏾',                            # 👆🏾 backhand index pointing up: medium-dark skin tone
                 '👆🏿')),                          # 👆🏿 backhand index pointing up: dark skin tone
            ('🖕',                                 # 🖕 middle finger
                ('🖕🏻',                            # 🖕🏻 middle finger: light skin tone
                 '🖕🏼',                            # 🖕🏼 middle finger: medium-light skin tone
                 '🖕🏽',                            # 🖕🏽 middle finger: medium skin tone
                 '🖕🏾',                            # 🖕🏾 middle finger: medium-dark skin tone
                 '🖕🏿')),                          # 🖕🏿 middle finger: dark skin tone
            ('👇',                                 # 👇 backhand index pointing down
                ('👇🏻',                            # 👇🏻 backhand index pointing down: light skin tone
                 '👇🏼',                            # 👇🏼 backhand index pointing down: medium-light skin tone
                 '👇🏽',                            # 👇🏽 backhand index pointing down: medium skin tone
                 '👇🏾',                            # 👇🏾 backhand index pointing down: medium-dark skin tone
                 '👇🏿')),                          # 👇🏿 backhand index pointing down: dark skin tone
            ('✌',                                 # ✌ victory hand
                ('✌🏻',                            # ✌🏻 victory hand: light skin tone
                 '✌🏼',                            # ✌🏼 victory hand: medium-light skin tone
                 '✌🏽',                            # ✌🏽 victory hand: medium skin tone
                 '✌🏾',                            # ✌🏾 victory hand: medium-dark skin tone
                 '✌🏿')),                          # ✌🏿 victory hand: dark skin tone
            ('\U0001f91e',                        # 🤞 crossed fingers
                ('\U0001f91e🏻',                   # 🤞🏻 crossed fingers: light skin tone
                 '\U0001f91e🏼',                   # 🤞🏼 crossed fingers: medium-light skin tone
                 '\U0001f91e🏽',                   # 🤞🏽 crossed fingers: medium skin tone
                 '\U0001f91e🏾',                   # 🤞🏾 crossed fingers: medium-dark skin tone
                 '\U0001f91e🏿')),                 # 🤞🏿 crossed fingers: dark skin tone
            ('🖖',                                 # 🖖 vulcan salute
                ('🖖🏻',                            # 🖖🏻 vulcan salute: light skin tone
                 '🖖🏼',                            # 🖖🏼 vulcan salute: medium-light skin tone
                 '🖖🏽',                            # 🖖🏽 vulcan salute: medium skin tone
                 '🖖🏾',                            # 🖖🏾 vulcan salute: medium-dark skin tone
                 '🖖🏿')),                          # 🖖🏿 vulcan salute: dark skin tone
            ('🤘',                                 # 🤘 sign of the horns
                ('🤘🏻',                            # 🤘🏻 sign of the horns: light skin tone
                 '🤘🏼',                            # 🤘🏼 sign of the horns: medium-light skin tone
                 '🤘🏽',                            # 🤘🏽 sign of the horns: medium skin tone
                 '🤘🏾',                            # 🤘🏾 sign of the horns: medium-dark skin tone
                 '🤘🏿')),                          # 🤘🏿 sign of the horns: dark skin tone
            ('\U0001f919',                        # 🤙 call me hand
                ('\U0001f919🏻',                   # 🤙🏻 call me hand: light skin tone
                 '\U0001f919🏼',                   # 🤙🏼 call me hand: medium-light skin tone
                 '\U0001f919🏽',                   # 🤙🏽 call me hand: medium skin tone
                 '\U0001f919🏾',                   # 🤙🏾 call me hand: medium-dark skin tone
                 '\U0001f919🏿')),                 # 🤙🏿 call me hand: dark skin tone
            ('🖐',                                 # 🖐 raised hand with fingers splayed
                ('🖐🏻',                            # 🖐🏻 raised hand with fingers splayed: light skin tone
                 '🖐🏼',                            # 🖐🏼 raised hand with fingers splayed: medium-light skin tone
                 '🖐🏽',                            # 🖐🏽 raised hand with fingers splayed: medium skin tone
                 '🖐🏾',                            # 🖐🏾 raised hand with fingers splayed: medium-dark skin tone
                 '🖐🏿')),                          # 🖐🏿 raised hand with fingers splayed: dark skin tone
            ('✋',                                 # ✋ raised hand
                ('✋🏻',                            # ✋🏻 raised hand: light skin tone
                 '✋🏼',                            # ✋🏼 raised hand: medium-light skin tone
                 '✋🏽',                            # ✋🏽 raised hand: medium skin tone
                 '✋🏾',                            # ✋🏾 raised hand: medium-dark skin tone
                 '✋🏿')),                          # ✋🏿 raised hand: dark skin tone
            ('👌',                                 # 👌 OK hand
                ('👌🏻',                            # 👌🏻 OK hand: light skin tone
                 '👌🏼',                            # 👌🏼 OK hand: medium-light skin tone
                 '👌🏽',                            # 👌🏽 OK hand: medium skin tone
                 '👌🏾',                            # 👌🏾 OK hand: medium-dark skin tone
                 '👌🏿')),                          # 👌🏿 OK hand: dark skin tone
            ('👍',                                 # 👍 thumbs up
                ('👍🏻',                            # 👍🏻 thumbs up: light skin tone
                 '👍🏼',                            # 👍🏼 thumbs up: medium-light skin tone
                 '👍🏽',                            # 👍🏽 thumbs up: medium skin tone
                 '👍🏾',                            # 👍🏾 thumbs up: medium-dark skin tone
                 '👍🏿')),                          # 👍🏿 thumbs up: dark skin tone
            ('👎',                                 # 👎 thumbs down
                ('👎🏻',                            # 👎🏻 thumbs down: light skin tone
                 '👎🏼',                            # 👎🏼 thumbs down: medium-light skin tone
                 '👎🏽',                            # 👎🏽 thumbs down: medium skin tone
                 '👎🏾',                            # 👎🏾 thumbs down: medium-dark skin tone
                 '👎🏿')),                          # 👎🏿 thumbs down: dark skin tone
            ('✊',                                 # ✊ raised fist
                ('✊🏻',                            # ✊🏻 raised fist: light skin tone
                 '✊🏼',                            # ✊🏼 raised fist: medium-light skin tone
                 '✊🏽',                            # ✊🏽 raised fist: medium skin tone
                 '✊🏾',                            # ✊🏾 raised fist: medium-dark skin tone
                 '✊🏿')),                          # ✊🏿 raised fist: dark skin tone
            ('👊',                                 # 👊 oncoming fist
                ('👊🏻',                            # 👊🏻 oncoming fist: light skin tone
                 '👊🏼',                            # 👊🏼 oncoming fist: medium-light skin tone
                 '👊🏽',                            # 👊🏽 oncoming fist: medium skin tone
                 '👊🏾',                            # 👊🏾 oncoming fist: medium-dark skin tone
                 '👊🏿')),                          # 👊🏿 oncoming fist: dark skin tone
            ('\U0001f91b',                        # 🤛 left-facing fist
                ('\U0001f91b🏻',                   # 🤛🏻 left-facing fist: light skin tone
                 '\U0001f91b🏼',                   # 🤛🏼 left-facing fist: medium-light skin tone
                 '\U0001f91b🏽',                   # 🤛🏽 left-facing fist: medium skin tone
                 '\U0001f91b🏾',                   # 🤛🏾 left-facing fist: medium-dark skin tone
                 '\U0001f91b🏿')),                 # 🤛🏿 left-facing fist: dark skin tone
            ('\U0001f91c',                        # 🤜 right-facing fist
                ('\U0001f91c🏻',                   # 🤜🏻 right-facing fist: light skin tone
                 '\U0001f91c🏼',                   # 🤜🏼 right-facing fist: medium-light skin tone
                 '\U0001f91c🏽',                   # 🤜🏽 right-facing fist: medium skin tone
                 '\U0001f91c🏾',                   # 🤜🏾 right-facing fist: medium-dark skin tone
                 '\U0001f91c🏿')),                 # 🤜🏿 right-facing fist: dark skin tone
            ('\U0001f91a',                        # 🤚 raised back of hand
                ('\U0001f91a🏻',                   # 🤚🏻 raised back of hand: light skin tone
                 '\U0001f91a🏼',                   # 🤚🏼 raised back of hand: medium-light skin tone
                 '\U0001f91a🏽',                   # 🤚🏽 raised back of hand: medium skin tone
                 '\U0001f91a🏾',                   # 🤚🏾 raised back of hand: medium-dark skin tone
                 '\U0001f91a🏿')),                 # 🤚🏿 raised back of hand: dark skin tone
            ('👋',                                 # 👋 waving hand
                ('👋🏻',                            # 👋🏻 waving hand: light skin tone
                 '👋🏼',                            # 👋🏼 waving hand: medium-light skin tone
                 '👋🏽',                            # 👋🏽 waving hand: medium skin tone
                 '👋🏾',                            # 👋🏾 waving hand: medium-dark skin tone
                 '👋🏿')),                          # 👋🏿 waving hand: dark skin tone
            ('👏',                                 # 👏 clapping hands
                ('👏🏻',                            # 👏🏻 clapping hands: light skin tone
                 '👏🏼',                            # 👏🏼 clapping hands: medium-light skin tone
                 '👏🏽',                            # 👏🏽 clapping hands: medium skin tone
                 '👏🏾',                            # 👏🏾 clapping hands: medium-dark skin tone
                 '👏🏿')),                          # 👏🏿 clapping hands: dark skin tone
            ('✍',                                 # ✍ writing hand
                ('✍🏻',                            # ✍🏻 writing hand: light skin tone
                 '✍🏼',                            # ✍🏼 writing hand: medium-light skin tone
                 '✍🏽',                            # ✍🏽 writing hand: medium skin tone
                 '✍🏾',                            # ✍🏾 writing hand: medium-dark skin tone
                 '✍🏿')),                          # ✍🏿 writing hand: dark skin tone
            ('👐',                                 # 👐 open hands
                ('👐🏻',                            # 👐🏻 open hands: light skin tone
                 '👐🏼',                            # 👐🏼 open hands: medium-light skin tone
                 '👐🏽',                            # 👐🏽 open hands: medium skin tone
                 '👐🏾',                            # 👐🏾 open hands: medium-dark skin tone
                 '👐🏿')),                          # 👐🏿 open hands: dark skin tone
            ('🙌',                                 # 🙌 raising hands
                ('🙌🏻',                            # 🙌🏻 raising hands: light skin tone
                 '🙌🏼',                            # 🙌🏼 raising hands: medium-light skin tone
                 '🙌🏽',                            # 🙌🏽 raising hands: medium skin tone
                 '🙌🏾',                            # 🙌🏾 raising hands: medium-dark skin tone
                 '🙌🏿')),                          # 🙌🏿 raising hands: dark skin tone
            ('🙏',                                 # 🙏 folded hands
                ('🙏🏻',                            # 🙏🏻 folded hands: light skin tone
                 '🙏🏼',                            # 🙏🏼 folded hands: medium-light skin tone
                 '🙏🏽',                            # 🙏🏽 folded hands: medium skin tone
                 '🙏🏾',                            # 🙏🏾 folded hands: medium-dark skin tone
                 '🙏🏿')),                          # 🙏🏿 folded hands: dark skin tone
            ('\U0001f91d', None),                 # 🤝 handshake
            ('💅',                                 # 💅 nail polish
                ('💅🏻',                            # 💅🏻 nail polish: light skin tone
                 '💅🏼',                            # 💅🏼 nail polish: medium-light skin tone
                 '💅🏽',                            # 💅🏽 nail polish: medium skin tone
                 '💅🏾',                            # 💅🏾 nail polish: medium-dark skin tone
                 '💅🏿')),                          # 💅🏿 nail polish: dark skin tone
            ('👂',                                 # 👂 ear
                ('👂🏻',                            # 👂🏻 ear: light skin tone
                 '👂🏼',                            # 👂🏼 ear: medium-light skin tone
                 '👂🏽',                            # 👂🏽 ear: medium skin tone
                 '👂🏾',                            # 👂🏾 ear: medium-dark skin tone
                 '👂🏿')),                          # 👂🏿 ear: dark skin tone
            ('👃',                                 # 👃 nose
                ('👃🏻',                            # 👃🏻 nose: light skin tone
                 '👃🏼',                            # 👃🏼 nose: medium-light skin tone
                 '👃🏽',                            # 👃🏽 nose: medium skin tone
                 '👃🏾',                            # 👃🏾 nose: medium-dark skin tone
                 '👃🏿')),                          # 👃🏿 nose: dark skin tone
            ('👣', None),                          # 👣 footprints
            ('👀', None),                          # 👀 eyes
            ('👁', None),                          # 👁 eye
            ('👁️\u200d🗨️', None),                 # 👁️‍🗨️ eye in speech bubble
            ('👅', None),                          # 👅 tongue
            ('👄', None),                          # 👄 mouth
        ]],
    [0, '❤',                                      # category: emotion
        [
            ('💋', None),                          # 💋 kiss mark
            ('💘', None),                          # 💘 heart with arrow
            ('❤', None),                          # ❤ red heart
            ('💓', None),                          # 💓 beating heart
            ('💔', None),                          # 💔 broken heart
            ('💕', None),                          # 💕 two hearts
            ('💖', None),                          # 💖 sparkling heart
            ('💗', None),                          # 💗 growing heart
            ('💙', None),                          # 💙 blue heart
            ('💚', None),                          # 💚 green heart
            ('💛', None),                          # 💛 yellow heart
            ('💜', None),                          # 💜 purple heart
            ('\U0001f5a4', None),                 # 🖤 black heart
            ('💝', None),                          # 💝 heart with ribbon
            ('💞', None),                          # 💞 revolving hearts
            ('💟', None),                          # 💟 heart decoration
            ('❣', None),                          # ❣ heavy heart exclamation
            ('💌', None),                          # 💌 love letter
            ('💤', None),                          # 💤 zzz
            ('💢', None),                          # 💢 anger symbol
            ('💣', None),                          # 💣 bomb
            ('💥', None),                          # 💥 collision
            ('💦', None),                          # 💦 sweat droplets
            ('💨', None),                          # 💨 dashing away
            ('💫', None),                          # 💫 dizzy
            ('💬', None),                          # 💬 speech balloon
            ('🗨', None),                          # 🗨 left speech bubble
            ('🗯', None),                          # 🗯 right anger bubble
            ('💭', None),                          # 💭 thought balloon
            ('🕳', None),                          # 🕳 hole
        ]],
    [1, '👓',                                      # category: clothing
        [
            ('👓', None),                          # 👓 glasses
            ('🕶', None),                          # 🕶 sunglasses
            ('👔', None),                          # 👔 necktie
            ('👕', None),                          # 👕 t-shirt
            ('👖', None),                          # 👖 jeans
            ('👗', None),                          # 👗 dress
            ('👘', None),                          # 👘 kimono
            ('👙', None),                          # 👙 bikini
            ('👚', None),                          # 👚 woman’s clothes
            ('👛', None),                          # 👛 purse
            ('👜', None),                          # 👜 handbag
            ('👝', None),                          # 👝 clutch bag
            ('🛍', None),                          # 🛍 shopping bags
            ('🎒', None),                          # 🎒 school backpack
            ('👞', None),                          # 👞 man’s shoe
            ('👟', None),                          # 👟 running shoe
            ('👠', None),                          # 👠 high-heeled shoe
            ('👡', None),                          # 👡 woman’s sandal
            ('👢', None),                          # 👢 woman’s boot
            ('👑', None),                          # 👑 crown
            ('👒', None),                          # 👒 woman’s hat
            ('🎩', None),                          # 🎩 top hat
            ('🎓', None),                          # 🎓 graduation cap
            ('⛑', None),                          # ⛑ rescue worker’s helmet
            ('📿', None),                          # 📿 prayer beads
            ('💄', None),                          # 💄 lipstick
            ('💍', None),                          # 💍 ring
            ('💎', None),                          # 💎 gem stone
        ]],
    [0, '🐘',                                      # category: animal-mammal
        [
            ('🐵', None),                          # 🐵 monkey face
            ('🐒', None),                          # 🐒 monkey
            ('\U0001f98d', None),                 # 🦍 gorilla
            ('🐶', None),                          # 🐶 dog face
            ('🐕', None),                          # 🐕 dog
            ('🐩', None),                          # 🐩 poodle
            ('🐺', None),                          # 🐺 wolf face
            ('\U0001f98a', None),                 # 🦊 fox face
            ('🐱', None),                          # 🐱 cat face
            ('🐈', None),                          # 🐈 cat
            ('🦁', None),                          # 🦁 lion face
            ('🐯', None),                          # 🐯 tiger face
            ('🐅', None),                          # 🐅 tiger
            ('🐆', None),                          # 🐆 leopard
            ('🐴', None),                          # 🐴 horse face
            ('🐎', None),                          # 🐎 horse
            ('\U0001f98c', None),                 # 🦌 deer
            ('🦄', None),                          # 🦄 unicorn face
            ('🐮', None),                          # 🐮 cow face
            ('🐂', None),                          # 🐂 ox
            ('🐃', None),                          # 🐃 water buffalo
            ('🐄', None),                          # 🐄 cow
            ('🐷', None),                          # 🐷 pig face
            ('🐖', None),                          # 🐖 pig
            ('🐗', None),                          # 🐗 boar
            ('🐽', None),                          # 🐽 pig nose
            ('🐏', None),                          # 🐏 ram
            ('🐑', None),                          # 🐑 sheep
            ('🐐', None),                          # 🐐 goat
            ('🐪', None),                          # 🐪 camel
            ('🐫', None),                          # 🐫 two-hump camel
            ('🐘', None),                          # 🐘 elephant
            ('\U0001f98f', None),                 # 🦏 rhinoceros
            ('🐭', None),                          # 🐭 mouse face
            ('🐁', None),                          # 🐁 mouse
            ('🐀', None),                          # 🐀 rat
            ('🐹', None),                          # 🐹 hamster face
            ('🐰', None),                          # 🐰 rabbit face
            ('🐇', None),                          # 🐇 rabbit
            ('🐿', None),                          # 🐿 chipmunk
            ('\U0001f987', None),                 # 🦇 bat
            ('🐻', None),                          # 🐻 bear face
            ('🐨', None),                          # 🐨 koala
            ('🐼', None),                          # 🐼 panda face
            ('🐾', None),                          # 🐾 paw prints
        ]],
    [1, '🦃',                                      # category: animal-bird
        [
            ('🦃', None),                          # 🦃 turkey
            ('🐔', None),                          # 🐔 chicken
            ('🐓', None),                          # 🐓 rooster
            ('🐣', None),                          # 🐣 hatching chick
            ('🐤', None),                          # 🐤 baby chick
            ('🐥', None),                          # 🐥 front-facing baby chick
            ('🐦', None),                          # 🐦 bird
            ('🐧', None),                          # 🐧 penguin
            ('🕊', None),                          # 🕊 dove
            ('\U0001f985', None),                 # 🦅 eagle
            ('\U0001f986', None),                 # 🦆 duck
            ('\U0001f989', None),                 # 🦉 owl
        ]],
    [1, '🐸',                                      # category: animal-amphibian
        [
            ('🐸', None),                          # 🐸 frog face
            ('🐊', None),                          # 🐊 crocodile
            ('🐢', None),                          # 🐢 turtle
            ('\U0001f98e', None),                 # 🦎 lizard
            ('🐍', None),                          # 🐍 snake
            ('🐲', None),                          # 🐲 dragon face
            ('🐉', None),                          # 🐉 dragon
        ]],
    [1, '🐳',                                      # category: animal-marine
        [
            ('🐳', None),                          # 🐳 spouting whale
            ('🐋', None),                          # 🐋 whale
            ('🐬', None),                          # 🐬 dolphin
            ('🐟', None),                          # 🐟 fish
            ('🐠', None),                          # 🐠 tropical fish
            ('🐡', None),                          # 🐡 blowfish
            ('\U0001f988', None),                 # 🦈 shark
            ('🐙', None),                          # 🐙 octopus
            ('🐚', None),                          # 🐚 spiral shell
            ('🦀', None),                          # 🦀 crab
            ('\U0001f990', None),                 # 🦐 shrimp
            ('\U0001f991', None),                 # 🦑 squid
        ]],
    [1, '\U0001f98b',                             # category: animal-bug
        [
            ('\U0001f98b', None),                 # 🦋 butterfly
            ('🐌', None),                          # 🐌 snail
            ('🐛', None),                          # 🐛 bug
            ('🐜', None),                          # 🐜 ant
            ('🐝', None),                          # 🐝 honeybee
            ('🐞', None),                          # 🐞 lady beetle
            ('🕷', None),                          # 🕷 spider
            ('🕸', None),                          # 🕸 spider web
            ('🦂', None),                          # 🦂 scorpion
        ]],
    [1, '💐',                                      # category: plant-flower
        [
            ('💐', None),                          # 💐 bouquet
            ('🌸', None),                          # 🌸 cherry blossom
            ('💮', None),                          # 💮 white flower
            ('🏵', None),                          # 🏵 rosette
            ('🌹', None),                          # 🌹 rose
            ('\U0001f940', None),                 # 🥀 wilted flower
            ('🌺', None),                          # 🌺 hibiscus
            ('🌻', None),                          # 🌻 sunflower
            ('🌼', None),                          # 🌼 blossom
            ('🌷', None),                          # 🌷 tulip
        ]],
    [1, '🌱',                                      # category: plant-other
        [
            ('🌱', None),                          # 🌱 seedling
            ('🌲', None),                          # 🌲 evergreen tree
            ('🌳', None),                          # 🌳 deciduous tree
            ('🌴', None),                          # 🌴 palm tree
            ('🌵', None),                          # 🌵 cactus
            ('🌾', None),                          # 🌾 sheaf of rice
            ('🌿', None),                          # 🌿 herb
            ('☘', None),                          # ☘ shamrock
            ('🍀', None),                          # 🍀 four leaf clover
            ('🍁', None),                          # 🍁 maple leaf
            ('🍂', None),                          # 🍂 fallen leaf
            ('🍃', None),                          # 🍃 leaf fluttering in wind
        ]],
    [0, '🍉',                                      # category: food-fruit
        [
            ('🍇', None),                          # 🍇 grapes
            ('🍈', None),                          # 🍈 melon
            ('🍉', None),                          # 🍉 watermelon
            ('🍊', None),                          # 🍊 tangerine
            ('🍋', None),                          # 🍋 lemon
            ('🍌', None),                          # 🍌 banana
            ('🍍', None),                          # 🍍 pineapple
            ('🍎', None),                          # 🍎 red apple
            ('🍏', None),                          # 🍏 green apple
            ('🍐', None),                          # 🍐 pear
            ('🍑', None),                          # 🍑 peach
            ('🍒', None),                          # 🍒 cherries
            ('🍓', None),                          # 🍓 strawberry
            ('\U0001f95d', None),                 # 🥝 kiwi fruit
            ('🍅', None),                          # 🍅 tomato
        ]],
    [1, '\U0001f951',                             # category: food-vegetable
        [
            ('\U0001f951', None),                 # 🥑 avocado
            ('🍆', None),                          # 🍆 eggplant
            ('\U0001f954', None),                 # 🥔 potato
            ('\U0001f955', None),                 # 🥕 carrot
            ('🌽', None),                          # 🌽 ear of corn
            ('🌶', None),                          # 🌶 hot pepper
            ('\U0001f952', None),                 # 🥒 cucumber
            ('🍄', None),                          # 🍄 mushroom
            ('\U0001f95c', None),                 # 🥜 peanuts
            ('🌰', None),                          # 🌰 chestnut
        ]],
    [1, '🍞',                                      # category: food-prepared
        [
            ('🍞', None),                          # 🍞 bread
            ('\U0001f950', None),                 # 🥐 croissant
            ('\U0001f956', None),                 # 🥖 baguette bread
            ('\U0001f95e', None),                 # 🥞 pancakes
            ('🧀', None),                          # 🧀 cheese wedge
            ('🍖', None),                          # 🍖 meat on bone
            ('🍗', None),                          # 🍗 poultry leg
            ('\U0001f953', None),                 # 🥓 bacon
            ('🍔', None),                          # 🍔 hamburger
            ('🍟', None),                          # 🍟 french fries
            ('🍕', None),                          # 🍕 pizza
            ('🌭', None),                          # 🌭 hot dog
            ('🌮', None),                          # 🌮 taco
            ('🌯', None),                          # 🌯 burrito
            ('\U0001f959', None),                 # 🥙 stuffed flatbread
            ('\U0001f95a', None),                 # 🥚 egg
            ('🍳', None),                          # 🍳 cooking
            ('\U0001f958', None),                 # 🥘 shallow pan of food
            ('🍲', None),                          # 🍲 pot of food
            ('\U0001f957', None),                 # 🥗 green salad
            ('🍿', None),                          # 🍿 popcorn
        ]],
    [1, '🍱',                                      # category: food-asian
        [
            ('🍱', None),                          # 🍱 bento box
            ('🍘', None),                          # 🍘 rice cracker
            ('🍙', None),                          # 🍙 rice ball
            ('🍚', None),                          # 🍚 cooked rice
            ('🍛', None),                          # 🍛 curry rice
            ('🍜', None),                          # 🍜 steaming bowl
            ('🍝', None),                          # 🍝 spaghetti
            ('🍠', None),                          # 🍠 roasted sweet potato
            ('🍢', None),                          # 🍢 oden
            ('🍣', None),                          # 🍣 sushi
            ('🍤', None),                          # 🍤 fried shrimp
            ('🍥', None),                          # 🍥 fish cake with swirl
            ('🍡', None),                          # 🍡 dango
        ]],
    [1, '🍦',                                      # category: food-sweet
        [
            ('🍦', None),                          # 🍦 soft ice cream
            ('🍧', None),                          # 🍧 shaved ice
            ('🍨', None),                          # 🍨 ice cream
            ('🍩', None),                          # 🍩 doughnut
            ('🍪', None),                          # 🍪 cookie
            ('🎂', None),                          # 🎂 birthday cake
            ('🍰', None),                          # 🍰 shortcake
            ('🍫', None),                          # 🍫 chocolate bar
            ('🍬', None),                          # 🍬 candy
            ('🍭', None),                          # 🍭 lollipop
            ('🍮', None),                          # 🍮 custard
            ('🍯', None),                          # 🍯 honey pot
        ]],
    [1, '🍼',                                      # category: drink
        [
            ('🍼', None),                          # 🍼 baby bottle
            ('\U0001f95b', None),                 # 🥛 glass of milk
            ('☕', None),                          # ☕ hot beverage
            ('🍵', None),                          # 🍵 teacup without handle
            ('🍶', None),                          # 🍶 sake
            ('🍾', None),                          # 🍾 bottle with popping cork
            ('🍷', None),                          # 🍷 wine glass
            ('🍸', None),                          # 🍸 cocktail glass
            ('🍹', None),                          # 🍹 tropical drink
            ('🍺', None),                          # 🍺 beer mug
            ('🍻', None),                          # 🍻 clinking beer mugs
            ('\U0001f942', None),                 # 🥂 clinking glasses
            ('\U0001f943', None),                 # 🥃 tumbler glass
        ]],
    [1, '🍽',                                      # category: dishware
        [
            ('🍽', None),                          # 🍽 fork and knife with plate
            ('🍴', None),                          # 🍴 fork and knife
            ('\U0001f944', None),                 # 🥄 spoon
            ('🔪', None),                          # 🔪 kitchen knife
            ('🏺', None),                          # 🏺 amphora
        ]],
    [0, '🌍',                                      # category: place-map
        [
            ('🌍', None),                          # 🌍 globe showing Europe-Africa
            ('🌎', None),                          # 🌎 globe showing Americas
            ('🌏', None),                          # 🌏 globe showing Asia-Australia
            ('🌐', None),                          # 🌐 globe with meridians
            ('🗺', None),                          # 🗺 world map
            ('🗾', None),                          # 🗾 map of Japan
        ]],
    [1, '🏔',                                      # category: place-geographic
        [
            ('🏔', None),                          # 🏔 snow-capped mountain
            ('⛰', None),                          # ⛰ mountain
            ('🌋', None),                          # 🌋 volcano
            ('🗻', None),                          # 🗻 mount fuji
            ('🏕', None),                          # 🏕 camping
            ('🏖', None),                          # 🏖 beach with umbrella
            ('🏜', None),                          # 🏜 desert
            ('🏝', None),                          # 🏝 desert island
            ('🏞', None),                          # 🏞 national park
        ]],
    [1, '🏟',                                      # category: place-building
        [
            ('🏟', None),                          # 🏟 stadium
            ('🏛', None),                          # 🏛 classical building
            ('🏗', None),                          # 🏗 building construction
            ('🏘', None),                          # 🏘 house
            ('🏙', None),                          # 🏙 cityscape
            ('🏚', None),                          # 🏚 derelict house
            ('🏠', None),                          # 🏠 house
            ('🏡', None),                          # 🏡 house with garden
            ('🏢', None),                          # 🏢 office building
            ('🏣', None),                          # 🏣 Japanese post office
            ('🏤', None),                          # 🏤 post office
            ('🏥', None),                          # 🏥 hospital
            ('🏦', None),                          # 🏦 bank
            ('🏨', None),                          # 🏨 hotel
            ('🏩', None),                          # 🏩 love hotel
            ('🏪', None),                          # 🏪 convenience store
            ('🏫', None),                          # 🏫 school
            ('🏬', None),                          # 🏬 department store
            ('🏭', None),                          # 🏭 factory
            ('🏯', None),                          # 🏯 Japanese castle
            ('🏰', None),                          # 🏰 castle
            ('💒', None),                          # 💒 wedding
            ('🗼', None),                          # 🗼 Tokyo tower
            ('🗽', None),                          # 🗽 Statue of Liberty
        ]],
    [1, '⛪',                                      # category: place-religious
        [
            ('⛪', None),                          # ⛪ church
            ('🕌', None),                          # 🕌 mosque
            ('🕍', None),                          # 🕍 synagogue
            ('⛩', None),                          # ⛩ shinto shrine
            ('🕋', None),                          # 🕋 kaaba
        ]],
    [1, '⛲',                                      # category: place-other
        [
            ('⛲', None),                          # ⛲ fountain
            ('⛺', None),                          # ⛺ tent
            ('🌁', None),                          # 🌁 foggy
            ('🌃', None),                          # 🌃 night with stars
            ('🌄', None),                          # 🌄 sunrise over mountains
            ('🌅', None),                          # 🌅 sunrise
            ('🌆', None),                          # 🌆 cityscape at dusk
            ('🌇', None),                          # 🌇 sunset
            ('🌉', None),                          # 🌉 bridge at night
            ('♨', None),                          # ♨ hot springs
            ('🌌', None),                          # 🌌 milky way
            ('🎠', None),                          # 🎠 carousel horse
            ('🎡', None),                          # 🎡 ferris wheel
            ('🎢', None),                          # 🎢 roller coaster
            ('💈', None),                          # 💈 barber pole
            ('🎪', None),                          # 🎪 circus tent
            ('🎭', None),                          # 🎭 performing arts
            ('🖼', None),                          # 🖼 framed picture
            ('🎨', None),                          # 🎨 artist palette
            ('🎰', None),                          # 🎰 slot machine
        ]],
    [1, '🚂',                                      # category: transport-ground
        [
            ('🚂', None),                          # 🚂 locomotive
            ('🚃', None),                          # 🚃 railway car
            ('🚄', None),                          # 🚄 high-speed train
            ('🚅', None),                          # 🚅 high-speed train with bullet nose
            ('🚆', None),                          # 🚆 train
            ('🚇', None),                          # 🚇 metro
            ('🚈', None),                          # 🚈 light rail
            ('🚉', None),                          # 🚉 station
            ('🚊', None),                          # 🚊 tram
            ('🚝', None),                          # 🚝 monorail
            ('🚞', None),                          # 🚞 mountain railway
            ('🚋', None),                          # 🚋 tram car
            ('🚌', None),                          # 🚌 bus
            ('🚍', None),                          # 🚍 oncoming bus
            ('🚎', None),                          # 🚎 trolleybus
            ('🚐', None),                          # 🚐 minibus
            ('🚑', None),                          # 🚑 ambulance
            ('🚒', None),                          # 🚒 fire engine
            ('🚓', None),                          # 🚓 police car
            ('🚔', None),                          # 🚔 oncoming police car
            ('🚕', None),                          # 🚕 taxi
            ('🚖', None),                          # 🚖 oncoming taxi
            ('🚗', None),                          # 🚗 automobile
            ('🚘', None),                          # 🚘 oncoming automobile
            ('🚙', None),                          # 🚙 sport utility vehicle
            ('🚚', None),                          # 🚚 delivery truck
            ('🚛', None),                          # 🚛 articulated lorry
            ('🚜', None),                          # 🚜 tractor
            ('🚲', None),                          # 🚲 bicycle
            ('\U0001f6f4', None),                 # 🛴 kick scooter
            ('\U0001f6f5', None),                 # 🛵 motor scooter
            ('🚏', None),                          # 🚏 bus stop
            ('🛣', None),                          # 🛣 motorway
            ('🛤', None),                          # 🛤 railway track
            ('⛽', None),                          # ⛽ fuel pump
            ('🚨', None),                          # 🚨 police car light
            ('🚥', None),                          # 🚥 horizontal traffic light
            ('🚦', None),                          # 🚦 vertical traffic light
            ('🚧', None),                          # 🚧 construction
            ('\U0001f6d1', None),                 # 🛑 stop sign
        ]],
    [1, '⚓',                                      # category: transport-water
        [
            ('⚓', None),                          # ⚓ anchor
            ('⛵', None),                          # ⛵ sailboat
            ('\U0001f6f6', None),                 # 🛶 canoe
            ('🚤', None),                          # 🚤 speedboat
            ('🛳', None),                          # 🛳 passenger ship
            ('⛴', None),                          # ⛴ ferry
            ('🛥', None),                          # 🛥 motor boat
            ('🚢', None),                          # 🚢 ship
        ]],
    [1, '✈',                                      # category: transport-air
        [
            ('✈', None),                          # ✈ airplane
            ('🛩', None),                          # 🛩 small airplane
            ('🛫', None),                          # 🛫 airplane departure
            ('🛬', None),                          # 🛬 airplane arrival
            ('💺', None),                          # 💺 seat
            ('🚁', None),                          # 🚁 helicopter
            ('🚟', None),                          # 🚟 suspension railway
            ('🚠', None),                          # 🚠 mountain cableway
            ('🚡', None),                          # 🚡 aerial tramway
            ('🚀', None),                          # 🚀 rocket
            ('🛰', None),                          # 🛰 satellite
        ]],
    [1, '🛎',                                      # category: hotel
        [
            ('🛎', None),                          # 🛎 bellhop bell
            ('🚪', None),                          # 🚪 door
            ('🛌',                                 # 🛌 person in bed
                ('🛌🏻',                            # 🛌🏻 person in bed: light skin tone
                 '🛌🏼',                            # 🛌🏼 person in bed: medium-light skin tone
                 '🛌🏽',                            # 🛌🏽 person in bed: medium skin tone
                 '🛌🏾',                            # 🛌🏾 person in bed: medium-dark skin tone
                 '🛌🏿')),                          # 🛌🏿 person in bed: dark skin tone
            ('🛏', None),                          # 🛏 bed
            ('🛋', None),                          # 🛋 couch and lamp
            ('🚽', None),                          # 🚽 toilet
            ('🚿', None),                          # 🚿 shower
            ('🛀',                                 # 🛀 person taking bath
                ('🛀🏻',                            # 🛀🏻 person taking bath: light skin tone
                 '🛀🏼',                            # 🛀🏼 person taking bath: medium-light skin tone
                 '🛀🏽',                            # 🛀🏽 person taking bath: medium skin tone
                 '🛀🏾',                            # 🛀🏾 person taking bath: medium-dark skin tone
                 '🛀🏿')),                          # 🛀🏿 person taking bath: dark skin tone
            ('🛁', None),                          # 🛁 bathtub
        ]],
    [1, '⌛',                                      # category: time
        [
            ('⌛', None),                          # ⌛ hourglass
            ('⏳', None),                          # ⏳ hourglass with flowing sand
            ('⌚', None),                          # ⌚ watch
            ('⏰', None),                          # ⏰ alarm clock
            ('⏱', None),                          # ⏱ stopwatch
            ('⏲', None),                          # ⏲ timer clock
            ('🕰', None),                          # 🕰 mantelpiece clock
            ('🕛', None),                          # 🕛 twelve o’clock
            ('🕧', None),                          # 🕧 twelve-thirty
            ('🕐', None),                          # 🕐 one o’clock
            ('🕜', None),                          # 🕜 one-thirty
            ('🕑', None),                          # 🕑 two o’clock
            ('🕝', None),                          # 🕝 two-thirty
            ('🕒', None),                          # 🕒 three o’clock
            ('🕞', None),                          # 🕞 three-thirty
            ('🕓', None),                          # 🕓 four o’clock
            ('🕟', None),                          # 🕟 four-thirty
            ('🕔', None),                          # 🕔 five o’clock
            ('🕠', None),                          # 🕠 five-thirty
            ('🕕', None),                          # 🕕 six o’clock
            ('🕡', None),                          # 🕡 six-thirty
            ('🕖', None),                          # 🕖 seven o’clock
            ('🕢', None),                          # 🕢 seven-thirty
            ('🕗', None),                          # 🕗 eight o’clock
            ('🕣', None),                          # 🕣 eight-thirty
            ('🕘', None),                          # 🕘 nine o’clock
            ('🕤', None),                          # 🕤 nine-thirty
            ('🕙', None),                          # 🕙 ten o’clock
            ('🕥', None),                          # 🕥 ten-thirty
            ('🕚', None),                          # 🕚 eleven o’clock
            ('🕦', None),                          # 🕦 eleven-thirty
        ]],
    [1, '🌑',                                      # category: sky & weather
        [
            ('🌑', None),                          # 🌑 new moon
            ('🌒', None),                          # 🌒 waxing crescent moon
            ('🌓', None),                          # 🌓 first quarter moon
            ('🌔', None),                          # 🌔 waxing gibbous moon
            ('🌕', None),                          # 🌕 full moon
            ('🌖', None),                          # 🌖 waning gibbous moon
            ('🌗', None),                          # 🌗 last quarter moon
            ('🌘', None),                          # 🌘 waning crescent moon
            ('🌙', None),                          # 🌙 crescent moon
            ('🌚', None),                          # 🌚 new moon face
            ('🌛', None),                          # 🌛 first quarter moon with face
            ('🌜', None),                          # 🌜 last quarter moon with face
            ('🌡', None),                          # 🌡 thermometer
            ('☀', None),                          # ☀ sun
            ('🌝', None),                          # 🌝 full moon with face
            ('🌞', None),                          # 🌞 sun with face
            ('⭐', None),                          # ⭐ white medium star
            ('🌟', None),                          # 🌟 glowing star
            ('🌠', None),                          # 🌠 shooting star
            ('☁', None),                          # ☁ cloud
            ('⛅', None),                          # ⛅ sun behind cloud
            ('⛈', None),                          # ⛈ cloud with lightning and rain
            ('🌤', None),                          # 🌤 sun behind small cloud
            ('🌥', None),                          # 🌥 sun behind large cloud
            ('🌦', None),                          # 🌦 sun behind rain cloud
            ('🌧', None),                          # 🌧 cloud with rain
            ('🌨', None),                          # 🌨 cloud with snow
            ('🌩', None),                          # 🌩 cloud with lightning
            ('🌪', None),                          # 🌪 tornado
            ('🌫', None),                          # 🌫 fog
            ('🌬', None),                          # 🌬 wind face
            ('🌀', None),                          # 🌀 cyclone
            ('🌈', None),                          # 🌈 rainbow
            ('🌂', None),                          # 🌂 closed umbrella
            ('☂', None),                          # ☂ umbrella
            ('☔', None),                          # ☔ umbrella with rain drops
            ('⛱', None),                          # ⛱ umbrella on ground
            ('⚡', None),                          # ⚡ high voltage
            ('❄', None),                          # ❄ snowflake
            ('☃', None),                          # ☃ snowman
            ('⛄', None),                          # ⛄ snowman without snow
            ('☄', None),                          # ☄ comet
            ('🔥', None),                          # 🔥 fire
            ('💧', None),                          # 💧 droplet
            ('🌊', None),                          # 🌊 water wave
        ]],
    [0, '✨',                                      # category: event
        [
            ('🎃', None),                          # 🎃 jack-o-lantern
            ('🎄', None),                          # 🎄 Christmas tree
            ('🎆', None),                          # 🎆 fireworks
            ('🎇', None),                          # 🎇 sparkler
            ('✨', None),                          # ✨ sparkles
            ('🎈', None),                          # 🎈 balloon
            ('🎉', None),                          # 🎉 party popper
            ('🎊', None),                          # 🎊 confetti ball
            ('🎋', None),                          # 🎋 tanabata tree
            ('🎍', None),                          # 🎍 pine decoration
            ('🎎', None),                          # 🎎 Japanese dolls
            ('🎏', None),                          # 🎏 carp streamer
            ('🎐', None),                          # 🎐 wind chime
            ('🎑', None),                          # 🎑 moon viewing ceremony
            ('🎀', None),                          # 🎀 ribbon
            ('🎁', None),                          # 🎁 wrapped gift
            ('🎗', None),                          # 🎗 reminder ribbon
            ('🎟', None),                          # 🎟 admission tickets
            ('🎫', None),                          # 🎫 ticket
        ]],
    [1, '🎖',                                      # category: award-medal
        [
            ('🎖', None),                          # 🎖 military medal
            ('🏆', None),                          # 🏆 trophy
            ('🏅', None),                          # 🏅 sports medal
            ('\U0001f947', None),                 # 🥇 1st place medal
            ('\U0001f948', None),                 # 🥈 2nd place medal
            ('\U0001f949', None),                 # 🥉 3rd place medal
        ]],
    [1, '⚽',                                      # category: sport
        [
            ('⚽', None),                          # ⚽ soccer ball
            ('⚾', None),                          # ⚾ baseball
            ('🏀', None),                          # 🏀 basketball
            ('🏐', None),                          # 🏐 volleyball
            ('🏈', None),                          # 🏈 american football
            ('🏉', None),                          # 🏉 rugby football
            ('🎾', None),                          # 🎾 tennis
            ('🎱', None),                          # 🎱 pool 8 ball
            ('🎳', None),                          # 🎳 bowling
            ('🏏', None),                          # 🏏 cricket
            ('🏑', None),                          # 🏑 field hockey
            ('🏒', None),                          # 🏒 ice hockey
            ('🏓', None),                          # 🏓 ping pong
            ('🏸', None),                          # 🏸 badminton
            ('\U0001f94a', None),                 # 🥊 boxing glove
            ('\U0001f94b', None),                 # 🥋 martial arts uniform
            ('\U0001f945', None),                 # 🥅 goal net
            ('🎯', None),                          # 🎯 direct hit
            ('⛳', None),                          # ⛳ flag in hole
            ('⛸', None),                          # ⛸ ice skate
            ('🎣', None),                          # 🎣 fishing pole
            ('🎽', None),                          # 🎽 running shirt
            ('🎿', None),                          # 🎿 skis
        ]],
    [1, '🎮',                                      # category: game
        [
            ('🎮', None),                          # 🎮 video game
            ('🕹', None),                          # 🕹 joystick
            ('🎲', None),                          # 🎲 game die
            ('♠', None),                          # ♠ spade suit
            ('♥', None),                          # ♥ heart suit
            ('♦', None),                          # ♦ diamond suit
            ('♣', None),                          # ♣ club suit
            ('🃏', None),                          # 🃏 joker
            ('🀄', None),                          # 🀄 mahjong red dragon
            ('🎴', None),                          # 🎴 flower playing cards
        ]],
    [0, '💡',                                      # category: sound
        [
            ('🔇', None),                          # 🔇 muted speaker
            ('🔈', None),                          # 🔈 speaker low volume
            ('🔉', None),                          # 🔉 speaker medium volume
            ('🔊', None),                          # 🔊 speaker high volume
            ('📢', None),                          # 📢 loudspeaker
            ('📣', None),                          # 📣 megaphone
            ('📯', None),                          # 📯 postal horn
            ('🔔', None),                          # 🔔 bell
            ('🔕', None),                          # 🔕 bell with slash
        ]],
    [1, '🎼',                                      # category: music
        [
            ('🎼', None),                          # 🎼 musical score
            ('🎵', None),                          # 🎵 musical note
            ('🎶', None),                          # 🎶 musical notes
            ('🎙', None),                          # 🎙 studio microphone
            ('🎚', None),                          # 🎚 level slider
            ('🎛', None),                          # 🎛 control knobs
            ('🎤', None),                          # 🎤 microphone
            ('🎧', None),                          # 🎧 headphone
            ('📻', None),                          # 📻 radio
        ]],
    [1, '🎷',                                      # category: musical-instrument
        [
            ('🎷', None),                          # 🎷 saxophone
            ('🎸', None),                          # 🎸 guitar
            ('🎹', None),                          # 🎹 musical keyboard
            ('🎺', None),                          # 🎺 trumpet
            ('🎻', None),                          # 🎻 violin
            ('\U0001f941', None),                 # 🥁 drum
        ]],
    [1, '📱',                                      # category: phone
        [
            ('📱', None),                          # 📱 mobile phone
            ('📲', None),                          # 📲 mobile phone with arrow
            ('☎', None),                          # ☎ telephone
            ('📞', None),                          # 📞 telephone receiver
            ('📟', None),                          # 📟 pager
            ('📠', None),                          # 📠 fax machine
        ]],
    [1, '🔋',                                      # category: computer
        [
            ('🔋', None),                          # 🔋 battery
            ('🔌', None),                          # 🔌 electric plug
            ('💻', None),                          # 💻 laptop computer
            ('🖥', None),                          # 🖥 desktop computer
            ('🖨', None),                          # 🖨 printer
            ('⌨', None),                          # ⌨ keyboard
            ('🖱', None),                          # 🖱 computer mouse
            ('🖲', None),                          # 🖲 trackball
            ('💽', None),                          # 💽 computer disk
            ('💾', None),                          # 💾 floppy disk
            ('💿', None),                          # 💿 optical disk
            ('📀', None),                          # 📀 dvd
        ]],
    [1, '🎥',                                      # category: light & video
        [
            ('🎥', None),                          # 🎥 movie camera
            ('🎞', None),                          # 🎞 film frames
            ('📽', None),                          # 📽 film projector
            ('🎬', None),                          # 🎬 clapper board
            ('📺', None),                          # 📺 television
            ('📷', None),                          # 📷 camera
            ('📸', None),                          # 📸 camera with flash
            ('📹', None),                          # 📹 video camera
            ('📼', None),                          # 📼 videocassette
            ('🔍', None),                          # 🔍 left-pointing magnifying glass
            ('🔎', None),                          # 🔎 right-pointing magnifying glass
            ('🔬', None),                          # 🔬 microscope
            ('🔭', None),                          # 🔭 telescope
            ('📡', None),                          # 📡 satellite antenna
            ('🕯', None),                          # 🕯 candle
            ('💡', None),                          # 💡 light bulb
            ('🔦', None),                          # 🔦 flashlight
            ('🏮', None),                          # 🏮 red paper lantern
        ]],
    [1, '📔',                                      # category: book-paper
        [
            ('📔', None),                          # 📔 notebook with decorative cover
            ('📕', None),                          # 📕 closed book
            ('📖', None),                          # 📖 open book
            ('📗', None),                          # 📗 green book
            ('📘', None),                          # 📘 blue book
            ('📙', None),                          # 📙 orange book
            ('📚', None),                          # 📚 books
            ('📓', None),                          # 📓 notebook
            ('📒', None),                          # 📒 ledger
            ('📃', None),                          # 📃 page with curl
            ('📜', None),                          # 📜 scroll
            ('📄', None),                          # 📄 page facing up
            ('📰', None),                          # 📰 newspaper
            ('🗞', None),                          # 🗞 rolled-up newspaper
            ('📑', None),                          # 📑 bookmark tabs
            ('🔖', None),                          # 🔖 bookmark
            ('🏷', None),                          # 🏷 label
        ]],
    [1, '💰',                                      # category: money
        [
            ('💰', None),                          # 💰 money bag
            ('💴', None),                          # 💴 yen banknote
            ('💵', None),                          # 💵 dollar banknote
            ('💶', None),                          # 💶 euro banknote
            ('💷', None),                          # 💷 pound banknote
            ('💸', None),                          # 💸 money with wings
            ('💳', None),                          # 💳 credit card
            ('💹', None),                          # 💹 chart increasing with yen
            ('💱', None),                          # 💱 currency exchange
            ('💲', None),                          # 💲 heavy dollar sign
        ]],
    [1, '✉',                                      # category: mail
        [
            ('✉', None),                          # ✉ envelope
            ('📧', None),                          # 📧 e-mail
            ('📨', None),                          # 📨 incoming envelope
            ('📩', None),                          # 📩 envelope with arrow
            ('📤', None),                          # 📤 outbox tray
            ('📥', None),                          # 📥 inbox tray
            ('📦', None),                          # 📦 package
            ('📫', None),                          # 📫 closed mailbox with raised flag
            ('📪', None),                          # 📪 closed mailbox with lowered flag
            ('📬', None),                          # 📬 open mailbox with raised flag
            ('📭', None),                          # 📭 open mailbox with lowered flag
            ('📮', None),                          # 📮 postbox
            ('🗳', None),                          # 🗳 ballot box with ballot
        ]],
    [1, '✏',                                      # category: writing
        [
            ('✏', None),                          # ✏ pencil
            ('✒', None),                          # ✒ black nib
            ('🖋', None),                          # 🖋 fountain pen
            ('🖊', None),                          # 🖊 pen
            ('🖌', None),                          # 🖌 paintbrush
            ('🖍', None),                          # 🖍 crayon
            ('📝', None),                          # 📝 memo
        ]],
    [1, '💼',                                      # category: office
        [
            ('💼', None),                          # 💼 briefcase
            ('📁', None),                          # 📁 file folder
            ('📂', None),                          # 📂 open file folder
            ('🗂', None),                          # 🗂 card index dividers
            ('📅', None),                          # 📅 calendar
            ('📆', None),                          # 📆 tear-off calendar
            ('🗒', None),                          # 🗒 spiral notepad
            ('🗓', None),                          # 🗓 spiral calendar
            ('📇', None),                          # 📇 card index
            ('📈', None),                          # 📈 chart increasing
            ('📉', None),                          # 📉 chart decreasing
            ('📊', None),                          # 📊 bar chart
            ('📋', None),                          # 📋 clipboard
            ('📌', None),                          # 📌 pushpin
            ('📍', None),                          # 📍 round pushpin
            ('📎', None),                          # 📎 paperclip
            ('🖇', None),                          # 🖇 linked paperclips
            ('📏', None),                          # 📏 straight ruler
            ('📐', None),                          # 📐 triangular ruler
            ('✂', None),                          # ✂ scissors
            ('🗃', None),                          # 🗃 card file box
            ('🗄', None),                          # 🗄 file cabinet
            ('🗑', None),                          # 🗑 wastebasket
        ]],
    [1, '🔒',                                      # category: lock
        [
            ('🔒', None),                          # 🔒 locked
            ('🔓', None),                          # 🔓 unlocked
            ('🔏', None),                          # 🔏 locked with pen
            ('🔐', None),                          # 🔐 locked with key
            ('🔑', None),                          # 🔑 key
            ('🗝', None),                          # 🗝 old key
        ]],
    [1, '🔨',                                      # category: tool
        [
            ('🔨', None),                          # 🔨 hammer
            ('⛏', None),                          # ⛏ pick
            ('⚒', None),                          # ⚒ hammer and pick
            ('🛠', None),                          # 🛠 hammer and wrench
            ('🗡', None),                          # 🗡 dagger
            ('⚔', None),                          # ⚔ crossed swords
            ('🔫', None),                          # 🔫 pistol
            ('🏹', None),                          # 🏹 bow and arrow
            ('🛡', None),                          # 🛡 shield
            ('🔧', None),                          # 🔧 wrench
            ('🔩', None),                          # 🔩 nut and bolt
            ('⚙', None),                          # ⚙ gear
            ('🗜', None),                          # 🗜 clamp
            ('⚗', None),                          # ⚗ alembic
            ('⚖', None),                          # ⚖ balance scale
            ('🔗', None),                          # 🔗 link
            ('⛓', None),                          # ⛓ chains
        ]],
    [1, '💉',                                      # category: medical
        [
            ('💉', None),                          # 💉 syringe
            ('💊', None),                          # 💊 pill
        ]],
    [1, '🚬',                                      # category: other-object
        [
            ('🚬', None),                          # 🚬 cigarette
            ('⚰', None),                          # ⚰ coffin
            ('⚱', None),                          # ⚱ funeral urn
            ('🗿', None),                          # 🗿 moai
            ('🛢', None),                          # 🛢 oil drum
            ('🔮', None),                          # 🔮 crystal ball
            ('\U0001f6d2', None),                 # 🛒 shopping cart
        ]],
    [0, '🔷',                                      # category: transport-sign
        [
            ('🏧', None),                          # 🏧 ATM sign
            ('🚮', None),                          # 🚮 litter in bin sign
            ('🚰', None),                          # 🚰 potable water
            ('♿', None),                          # ♿ wheelchair symbol
            ('🚹', None),                          # 🚹 men’s room
            ('🚺', None),                          # 🚺 women’s room
            ('🚻', None),                          # 🚻 restroom
            ('🚼', None),                          # 🚼 baby symbol
            ('🚾', None),                          # 🚾 water closet
            ('🛂', None),                          # 🛂 passport control
            ('🛃', None),                          # 🛃 customs
            ('🛄', None),                          # 🛄 baggage claim
            ('🛅', None),                          # 🛅 left luggage
        ]],
    [1, '⚠',                                      # category: warning
        [
            ('⚠', None),                          # ⚠ warning
            ('🚸', None),                          # 🚸 children crossing
            ('⛔', None),                          # ⛔ no entry
            ('🚫', None),                          # 🚫 prohibited
            ('🚳', None),                          # 🚳 no bicycles
            ('🚭', None),                          # 🚭 no smoking
            ('🚯', None),                          # 🚯 no littering
            ('🚱', None),                          # 🚱 non-potable water
            ('🚷', None),                          # 🚷 no pedestrians
            ('📵', None),                          # 📵 no mobile phones
            ('🔞', None),                          # 🔞 no one under eighteen
            ('☢', None),                          # ☢ radioactive
            ('☣', None),                          # ☣ biohazard
        ]],
    [1, '⬆',                                      # category: arrow
        [
            ('⬆', None),                          # ⬆ up arrow
            ('↗', None),                          # ↗ up-right arrow
            ('➡', None),                          # ➡ right arrow
            ('↘', None),                          # ↘ down-right arrow
            ('⬇', None),                          # ⬇ down arrow
            ('↙', None),                          # ↙ down-left arrow
            ('⬅', None),                          # ⬅ left arrow
            ('↖', None),                          # ↖ up-left arrow
            ('↕', None),                          # ↕ up-down arrow
            ('↔', None),                          # ↔ left-right arrow
            ('↩', None),                          # ↩ right arrow curving left
            ('↪', None),                          # ↪ left arrow curving right
            ('⤴', None),                          # ⤴ right arrow curving up
            ('⤵', None),                          # ⤵ right arrow curving down
            ('🔃', None),                          # 🔃 clockwise vertical arrows
            ('🔄', None),                          # 🔄 anticlockwise arrows button
            ('🔙', None),                          # 🔙 BACK arrow
            ('🔚', None),                          # 🔚 END arrow
            ('🔛', None),                          # 🔛 ON! arrow
            ('🔜', None),                          # 🔜 SOON arrow
            ('🔝', None),                          # 🔝 TOP arrow
        ]],
    [1, '🛐',                                      # category: religion
        [
            ('🛐', None),                          # 🛐 place of worship
            ('⚛', None),                          # ⚛ atom symbol
            ('🕉', None),                          # 🕉 om
            ('✡', None),                          # ✡ star of David
            ('☸', None),                          # ☸ wheel of dharma
            ('☯', None),                          # ☯ yin yang
            ('✝', None),                          # ✝ latin cross
            ('☦', None),                          # ☦ orthodox cross
            ('☪', None),                          # ☪ star and crescent
            ('☮', None),                          # ☮ peace symbol
            ('🕎', None),                          # 🕎 menorah
            ('🔯', None),                          # 🔯 dotted six-pointed star
        ]],
    [1, '♈',                                      # category: zodiac
        [
            ('♈', None),                          # ♈ Aries
            ('♉', None),                          # ♉ Taurus
            ('♊', None),                          # ♊ Gemini
            ('♋', None),                          # ♋ Cancer
            ('♌', None),                          # ♌ Leo
            ('♍', None),                          # ♍ Virgo
            ('♎', None),                          # ♎ Libra
            ('♏', None),                          # ♏ Scorpius
            ('♐', None),                          # ♐ Sagittarius
            ('♑', None),                          # ♑ Capricorn
            ('♒', None),                          # ♒ Aquarius
            ('♓', None),                          # ♓ Pisces
            ('⛎', None),                          # ⛎ Ophiuchus
        ]],
    [1, '🔀',                                      # category: av-symbol
        [
            ('🔀', None),                          # 🔀 shuffle tracks button
            ('🔁', None),                          # 🔁 repeat button
            ('🔂', None),                          # 🔂 repeat single button
            ('▶', None),                          # ▶ play button
            ('⏩', None),                          # ⏩ fast-forward button
            ('⏭', None),                          # ⏭ next track button
            ('⏯', None),                          # ⏯ play or pause button
            ('◀', None),                          # ◀ reverse button
            ('⏪', None),                          # ⏪ fast reverse button
            ('⏮', None),                          # ⏮ last track button
            ('🔼', None),                          # 🔼 up button
            ('⏫', None),                          # ⏫ fast up button
            ('🔽', None),                          # 🔽 down button
            ('⏬', None),                          # ⏬ fast down button
            ('⏸', None),                          # ⏸ pause button
            ('⏹', None),                          # ⏹ stop button
            ('⏺', None),                          # ⏺ record button
            ('⏏', None),                          # ⏏ eject button
            ('🎦', None),                          # 🎦 cinema
            ('🔅', None),                          # 🔅 dim button
            ('🔆', None),                          # 🔆 bright button
            ('📶', None),                          # 📶 antenna bars
            ('📳', None),                          # 📳 vibration mode
            ('📴', None),                          # 📴 mobile phone off
        ]],
    [1, '♻',                                      # category: other-symbol
        [
            ('♻', None),                          # ♻ recycling symbol
            ('📛', None),                          # 📛 name badge
            ('⚜', None),                          # ⚜ fleur-de-lis
            ('🔰', None),                          # 🔰 Japanese symbol for beginner
            ('🔱', None),                          # 🔱 trident emblem
            ('⭕', None),                          # ⭕ heavy large circle
            ('✅', None),                          # ✅ white heavy check mark
            ('☑', None),                          # ☑ ballot box with check
            ('✔', None),                          # ✔ heavy check mark
            ('✖', None),                          # ✖ heavy multiplication x
            ('❌', None),                          # ❌ cross mark
            ('❎', None),                          # ❎ cross mark button
            ('➕', None),                          # ➕ heavy plus sign
            ('➖', None),                          # ➖ heavy minus sign
            ('➗', None),                          # ➗ heavy division sign
            ('➰', None),                          # ➰ curly loop
            ('➿', None),                          # ➿ double curly loop
            ('〽', None),                          # 〽 part alternation mark
            ('✳', None),                          # ✳ eight-spoked asterisk
            ('✴', None),                          # ✴ eight-pointed star
            ('❇', None),                          # ❇ sparkle
            ('‼', None),                          # ‼ double exclamation mark
            ('⁉', None),                          # ⁉ exclamation question mark
            ('❓', None),                          # ❓ question mark
            ('❔', None),                          # ❔ white question mark
            ('❕', None),                          # ❕ white exclamation mark
            ('❗', None),                          # ❗ exclamation mark
            ('〰', None),                          # 〰 wavy dash
            ('©', None),                          # © copyright
            ('®', None),                          # ® registered
            ('™', None),                          # ™ trade mark
        ]],
    [1, '#️⃣',                                    # category: keycap
        [
            ('#️⃣', None),                        # #️⃣ keycap: #
            ('*️⃣', None),                        # *️⃣ keycap: *
            ('0️⃣', None),                        # 0️⃣ keycap: 0
            ('1️⃣', None),                        # 1️⃣ keycap: 1
            ('2️⃣', None),                        # 2️⃣ keycap: 2
            ('3️⃣', None),                        # 3️⃣ keycap: 3
            ('4️⃣', None),                        # 4️⃣ keycap: 4
            ('5️⃣', None),                        # 5️⃣ keycap: 5
            ('6️⃣', None),                        # 6️⃣ keycap: 6
            ('7️⃣', None),                        # 7️⃣ keycap: 7
            ('8️⃣', None),                        # 8️⃣ keycap: 8
            ('9️⃣', None),                        # 9️⃣ keycap: 9
            ('🔟', None),                          # 🔟 keycap 10
        ]],
    [1, '💯',                                      # category: alphanum
        [
            ('💯', None),                          # 💯 hundred points
            ('🔠', None),                          # 🔠 input latin uppercase
            ('🔡', None),                          # 🔡 input latin lowercase
            ('🔢', None),                          # 🔢 input numbers
            ('🔣', None),                          # 🔣 input symbols
            ('🔤', None),                          # 🔤 input latin letters
            ('🅰', None),                          # 🅰 A button (blood type)
            ('🆎', None),                          # 🆎 AB button (blood type)
            ('🅱', None),                          # 🅱 B button (blood type)
            ('🆑', None),                          # 🆑 CL button
            ('🆒', None),                          # 🆒 COOL button
            ('🆓', None),                          # 🆓 FREE button
            ('ℹ', None),                          # ℹ information
            ('🆔', None),                          # 🆔 ID button
            ('Ⓜ', None),                          # Ⓜ circled M
            ('🆕', None),                          # 🆕 NEW button
            ('🆖', None),                          # 🆖 NG button
            ('🅾', None),                          # 🅾 O button (blood type)
            ('🆗', None),                          # 🆗 OK button
            ('🅿', None),                          # 🅿 P button
            ('🆘', None),                          # 🆘 SOS button
            ('🆙', None),                          # 🆙 UP! button
            ('🆚', None),                          # 🆚 VS button
            ('🈁', None),                          # 🈁 Japanese “here” button
            ('🈂', None),                          # 🈂 Japanese “service charge” button
            ('🈷', None),                          # 🈷 Japanese “monthly amount” button
            ('🈶', None),                          # 🈶 Japanese “not free of charge” button
            ('🈯', None),                          # 🈯 Japanese “reserved” button
            ('🉐', None),                          # 🉐 Japanese “bargain” button
            ('🈹', None),                          # 🈹 Japanese “discount” button
            ('🈚', None),                          # 🈚 Japanese “free of charge” button
            ('🈲', None),                          # 🈲 Japanese “prohibited” button
            ('🉑', None),                          # 🉑 Japanese “acceptable” button
            ('🈸', None),                          # 🈸 Japanese “application” button
            ('🈴', None),                          # 🈴 Japanese “passing grade” button
            ('🈳', None),                          # 🈳 Japanese “vacancy” button
            ('㊗', None),                          # ㊗ Japanese “congratulations” button
            ('㊙', None),                          # ㊙ Japanese “secret” button
            ('🈺', None),                          # 🈺 Japanese “open for business” button
            ('🈵', None),                          # 🈵 Japanese “no vacancy” button
        ]],
    [1, '▪',                                      # category: geometric
        [
            ('▪', None),                          # ▪ black small square
            ('▫', None),                          # ▫ white small square
            ('◻', None),                          # ◻ white medium square
            ('◼', None),                          # ◼ black medium square
            ('◽', None),                          # ◽ white medium-small square
            ('◾', None),                          # ◾ black medium-small square
            ('⬛', None),                          # ⬛ black large square
            ('⬜', None),                          # ⬜ white large square
            ('🔶', None),                          # 🔶 large orange diamond
            ('🔷', None),                          # 🔷 large blue diamond
            ('🔸', None),                          # 🔸 small orange diamond
            ('🔹', None),                          # 🔹 small blue diamond
            ('🔺', None),                          # 🔺 red triangle pointed up
            ('🔻', None),                          # 🔻 red triangle pointed down
            ('💠', None),                          # 💠 diamond with a dot
            ('🔘', None),                          # 🔘 radio button
            ('🔲', None),                          # 🔲 black square button
            ('🔳', None),                          # 🔳 white square button
            ('⚪', None),                          # ⚪ white circle
            ('⚫', None),                          # ⚫ black circle
            ('🔴', None),                          # 🔴 red circle
            ('🔵', None),                          # 🔵 blue circle
        ]],
    [0, '🚩',                                      # category: flag
        [
            ('🏁', None),                          # 🏁 chequered flag
            ('🚩', None),                          # 🚩 triangular flag
            ('🎌', None),                          # 🎌 crossed flags
            ('🏴', None),                          # 🏴 black flag
            ('🏳', None),                          # 🏳 white flag
            ('🏳️\u200d🌈', None),                  # 🏳️‍🌈 rainbow flag
        ]],
    [1, '🇦🇨',                                     # category: country-flag
        [
            ('🇦🇨', None),                         # 🇦🇨 Ascension Island
            ('🇦🇩', None),                         # 🇦🇩 Andorra
            ('🇦🇪', None),                         # 🇦🇪 United Arab Emirates
            ('🇦🇫', None),                         # 🇦🇫 Afghanistan
            ('🇦🇬', None),                         # 🇦🇬 Antigua & Barbuda
            ('🇦🇮', None),                         # 🇦🇮 Anguilla
            ('🇦🇱', None),                         # 🇦🇱 Albania
            ('🇦🇲', None),                         # 🇦🇲 Armenia
            ('🇦🇴', None),                         # 🇦🇴 Angola
            ('🇦🇶', None),                         # 🇦🇶 Antarctica
            ('🇦🇷', None),                         # 🇦🇷 Argentina
            ('🇦🇸', None),                         # 🇦🇸 American Samoa
            ('🇦🇹', None),                         # 🇦🇹 Austria
            ('🇦🇺', None),                         # 🇦🇺 Australia
            ('🇦🇼', None),                         # 🇦🇼 Aruba
            ('🇦🇽', None),                         # 🇦🇽 Åland Islands
            ('🇦🇿', None),                         # 🇦🇿 Azerbaijan
            ('🇧🇦', None),                         # 🇧🇦 Bosnia & Herzegovina
            ('🇧🇧', None),                         # 🇧🇧 Barbados
            ('🇧🇩', None),                         # 🇧🇩 Bangladesh
            ('🇧🇪', None),                         # 🇧🇪 Belgium
            ('🇧🇫', None),                         # 🇧🇫 Burkina Faso
            ('🇧🇬', None),                         # 🇧🇬 Bulgaria
            ('🇧🇭', None),                         # 🇧🇭 Bahrain
            ('🇧🇮', None),                         # 🇧🇮 Burundi
            ('🇧🇯', None),                         # 🇧🇯 Benin
            ('🇧🇱', None),                         # 🇧🇱 St. Barthélemy
            ('🇧🇲', None),                         # 🇧🇲 Bermuda
            ('🇧🇳', None),                         # 🇧🇳 Brunei
            ('🇧🇴', None),                         # 🇧🇴 Bolivia
            ('🇧🇶', None),                         # 🇧🇶 Caribbean Netherlands
            ('🇧🇷', None),                         # 🇧🇷 Brazil
            ('🇧🇸', None),                         # 🇧🇸 Bahamas
            ('🇧🇹', None),                         # 🇧🇹 Bhutan
            ('🇧🇻', None),                         # 🇧🇻 Bouvet Island
            ('🇧🇼', None),                         # 🇧🇼 Botswana
            ('🇧🇾', None),                         # 🇧🇾 Belarus
            ('🇧🇿', None),                         # 🇧🇿 Belize
            ('🇨🇦', None),                         # 🇨🇦 Canada
            ('🇨🇨', None),                         # 🇨🇨 Cocos (Keeling) Islands
            ('🇨🇩', None),                         # 🇨🇩 Congo - Kinshasa
            ('🇨🇫', None),                         # 🇨🇫 Central African Republic
            ('🇨🇬', None),                         # 🇨🇬 Congo - Brazzaville
            ('🇨🇭', None),                         # 🇨🇭 Switzerland
            ('🇨🇮', None),                         # 🇨🇮 Côte d’Ivoire
            ('🇨🇰', None),                         # 🇨🇰 Cook Islands
            ('🇨🇱', None),                         # 🇨🇱 Chile
            ('🇨🇲', None),                         # 🇨🇲 Cameroon
            ('🇨🇳', None),                         # 🇨🇳 China
            ('🇨🇴', None),                         # 🇨🇴 Colombia
            ('🇨🇵', None),                         # 🇨🇵 Clipperton Island
            ('🇨🇷', None),                         # 🇨🇷 Costa Rica
            ('🇨🇺', None),                         # 🇨🇺 Cuba
            ('🇨🇻', None),                         # 🇨🇻 Cape Verde
            ('🇨🇼', None),                         # 🇨🇼 Curaçao
            ('🇨🇽', None),                         # 🇨🇽 Christmas Island
            ('🇨🇾', None),                         # 🇨🇾 Cyprus
            ('🇨🇿', None),                         # 🇨🇿 Czech Republic
            ('🇩🇪', None),                         # 🇩🇪 Germany
            ('🇩🇬', None),                         # 🇩🇬 Diego Garcia
            ('🇩🇯', None),                         # 🇩🇯 Djibouti
            ('🇩🇰', None),                         # 🇩🇰 Denmark
            ('🇩🇲', None),                         # 🇩🇲 Dominica
            ('🇩🇴', None),                         # 🇩🇴 Dominican Republic
            ('🇩🇿', None),                         # 🇩🇿 Algeria
            ('🇪🇦', None),                         # 🇪🇦 Ceuta & Melilla
            ('🇪🇨', None),                         # 🇪🇨 Ecuador
            ('🇪🇪', None),                         # 🇪🇪 Estonia
            ('🇪🇬', None),                         # 🇪🇬 Egypt
            ('🇪🇭', None),                         # 🇪🇭 Western Sahara
            ('🇪🇷', None),                         # 🇪🇷 Eritrea
            ('🇪🇸', None),                         # 🇪🇸 Spain
            ('🇪🇹', None),                         # 🇪🇹 Ethiopia
            ('🇪🇺', None),                         # 🇪🇺 European Union
            ('🇫🇮', None),                         # 🇫🇮 Finland
            ('🇫🇯', None),                         # 🇫🇯 Fiji
            ('🇫🇰', None),                         # 🇫🇰 Falkland Islands
            ('🇫🇲', None),                         # 🇫🇲 Micronesia
            ('🇫🇴', None),                         # 🇫🇴 Faroe Islands
            ('🇫🇷', None),                         # 🇫🇷 France
            ('🇬🇦', None),                         # 🇬🇦 Gabon
            ('🇬🇧', None),                         # 🇬🇧 United Kingdom
            ('🇬🇩', None),                         # 🇬🇩 Grenada
            ('🇬🇪', None),                         # 🇬🇪 Georgia
            ('🇬🇫', None),                         # 🇬🇫 French Guiana
            ('🇬🇬', None),                         # 🇬🇬 Guernsey
            ('🇬🇭', None),                         # 🇬🇭 Ghana
            ('🇬🇮', None),                         # 🇬🇮 Gibraltar
            ('🇬🇱', None),                         # 🇬🇱 Greenland
            ('🇬🇲', None),                         # 🇬🇲 Gambia
            ('🇬🇳', None),                         # 🇬🇳 Guinea
            ('🇬🇵', None),                         # 🇬🇵 Guadeloupe
            ('🇬🇶', None),                         # 🇬🇶 Equatorial Guinea
            ('🇬🇷', None),                         # 🇬🇷 Greece
            ('🇬🇸', None),                         # 🇬🇸 South Georgia & South Sandwich Islands
            ('🇬🇹', None),                         # 🇬🇹 Guatemala
            ('🇬🇺', None),                         # 🇬🇺 Guam
            ('🇬🇼', None),                         # 🇬🇼 Guinea-Bissau
            ('🇬🇾', None),                         # 🇬🇾 Guyana
            ('🇭🇰', None),                         # 🇭🇰 Hong Kong SAR China
            ('🇭🇲', None),                         # 🇭🇲 Heard & McDonald Islands
            ('🇭🇳', None),                         # 🇭🇳 Honduras
            ('🇭🇷', None),                         # 🇭🇷 Croatia
            ('🇭🇹', None),                         # 🇭🇹 Haiti
            ('🇭🇺', None),                         # 🇭🇺 Hungary
            ('🇮🇨', None),                         # 🇮🇨 Canary Islands
            ('🇮🇩', None),                         # 🇮🇩 Indonesia
            ('🇮🇪', None),                         # 🇮🇪 Ireland
            ('🇮🇱', None),                         # 🇮🇱 Israel
            ('🇮🇲', None),                         # 🇮🇲 Isle of Man
            ('🇮🇳', None),                         # 🇮🇳 India
            ('🇮🇴', None),                         # 🇮🇴 British Indian Ocean Territory
            ('🇮🇶', None),                         # 🇮🇶 Iraq
            ('🇮🇷', None),                         # 🇮🇷 Iran
            ('🇮🇸', None),                         # 🇮🇸 Iceland
            ('🇮🇹', None),                         # 🇮🇹 Italy
            ('🇯🇪', None),                         # 🇯🇪 Jersey
            ('🇯🇲', None),                         # 🇯🇲 Jamaica
            ('🇯🇴', None),                         # 🇯🇴 Jordan
            ('🇯🇵', None),                         # 🇯🇵 Japan
            ('🇰🇪', None),                         # 🇰🇪 Kenya
            ('🇰🇬', None),                         # 🇰🇬 Kyrgyzstan
            ('🇰🇭', None),                         # 🇰🇭 Cambodia
            ('🇰🇮', None),                         # 🇰🇮 Kiribati
            ('🇰🇲', None),                         # 🇰🇲 Comoros
            ('🇰🇳', None),                         # 🇰🇳 St. Kitts & Nevis
            ('🇰🇵', None),                         # 🇰🇵 North Korea
            ('🇰🇷', None),                         # 🇰🇷 South Korea
            ('🇰🇼', None),                         # 🇰🇼 Kuwait
            ('🇰🇾', None),                         # 🇰🇾 Cayman Islands
            ('🇰🇿', None),                         # 🇰🇿 Kazakhstan
            ('🇱🇦', None),                         # 🇱🇦 Laos
            ('🇱🇧', None),                         # 🇱🇧 Lebanon
            ('🇱🇨', None),                         # 🇱🇨 St. Lucia
            ('🇱🇮', None),                         # 🇱🇮 Liechtenstein
            ('🇱🇰', None),                         # 🇱🇰 Sri Lanka
            ('🇱🇷', None),                         # 🇱🇷 Liberia
            ('🇱🇸', None),                         # 🇱🇸 Lesotho
            ('🇱🇹', None),                         # 🇱🇹 Lithuania
            ('🇱🇺', None),                         # 🇱🇺 Luxembourg
            ('🇱🇻', None),                         # 🇱🇻 Latvia
            ('🇱🇾', None),                         # 🇱🇾 Libya
            ('🇲🇦', None),                         # 🇲🇦 Morocco
            ('🇲🇨', None),                         # 🇲🇨 Monaco
            ('🇲🇩', None),                         # 🇲🇩 Moldova
            ('🇲🇪', None),                         # 🇲🇪 Montenegro
            ('🇲🇫', None),                         # 🇲🇫 St. Martin
            ('🇲🇬', None),                         # 🇲🇬 Madagascar
            ('🇲🇭', None),                         # 🇲🇭 Marshall Islands
            ('🇲🇰', None),                         # 🇲🇰 Macedonia
            ('🇲🇱', None),                         # 🇲🇱 Mali
            ('🇲🇲', None),                         # 🇲🇲 Myanmar (Burma)
            ('🇲🇳', None),                         # 🇲🇳 Mongolia
            ('🇲🇴', None),                         # 🇲🇴 Macau SAR China
            ('🇲🇵', None),                         # 🇲🇵 Northern Mariana Islands
            ('🇲🇶', None),                         # 🇲🇶 Martinique
            ('🇲🇷', None),                         # 🇲🇷 Mauritania
            ('🇲🇸', None),                         # 🇲🇸 Montserrat
            ('🇲🇹', None),                         # 🇲🇹 Malta
            ('🇲🇺', None),                         # 🇲🇺 Mauritius
            ('🇲🇻', None),                         # 🇲🇻 Maldives
            ('🇲🇼', None),                         # 🇲🇼 Malawi
            ('🇲🇽', None),                         # 🇲🇽 Mexico
            ('🇲🇾', None),                         # 🇲🇾 Malaysia
            ('🇲🇿', None),                         # 🇲🇿 Mozambique
            ('🇳🇦', None),                         # 🇳🇦 Namibia
            ('🇳🇨', None),                         # 🇳🇨 New Caledonia
            ('🇳🇪', None),                         # 🇳🇪 Niger
            ('🇳🇫', None),                         # 🇳🇫 Norfolk Island
            ('🇳🇬', None),                         # 🇳🇬 Nigeria
            ('🇳🇮', None),                         # 🇳🇮 Nicaragua
            ('🇳🇱', None),                         # 🇳🇱 Netherlands
            ('🇳🇴', None),                         # 🇳🇴 Norway
            ('🇳🇵', None),                         # 🇳🇵 Nepal
            ('🇳🇷', None),                         # 🇳🇷 Nauru
            ('🇳🇺', None),                         # 🇳🇺 Niue
            ('🇳🇿', None),                         # 🇳🇿 New Zealand
            ('🇴🇲', None),                         # 🇴🇲 Oman
            ('🇵🇦', None),                         # 🇵🇦 Panama
            ('🇵🇪', None),                         # 🇵🇪 Peru
            ('🇵🇫', None),                         # 🇵🇫 French Polynesia
            ('🇵🇬', None),                         # 🇵🇬 Papua New Guinea
            ('🇵🇭', None),                         # 🇵🇭 Philippines
            ('🇵🇰', None),                         # 🇵🇰 Pakistan
            ('🇵🇱', None),                         # 🇵🇱 Poland
            ('🇵🇲', None),                         # 🇵🇲 St. Pierre & Miquelon
            ('🇵🇳', None),                         # 🇵🇳 Pitcairn Islands
            ('🇵🇷', None),                         # 🇵🇷 Puerto Rico
            ('🇵🇸', None),                         # 🇵🇸 Palestinian Territories
            ('🇵🇹', None),                         # 🇵🇹 Portugal
            ('🇵🇼', None),                         # 🇵🇼 Palau
            ('🇵🇾', None),                         # 🇵🇾 Paraguay
            ('🇶🇦', None),                         # 🇶🇦 Qatar
            ('🇷🇪', None),                         # 🇷🇪 Réunion
            ('🇷🇴', None),                         # 🇷🇴 Romania
            ('🇷🇸', None),                         # 🇷🇸 Serbia
            ('🇷🇺', None),                         # 🇷🇺 Russia
            ('🇷🇼', None),                         # 🇷🇼 Rwanda
            ('🇸🇦', None),                         # 🇸🇦 Saudi Arabia
            ('🇸🇧', None),                         # 🇸🇧 Solomon Islands
            ('🇸🇨', None),                         # 🇸🇨 Seychelles
            ('🇸🇩', None),                         # 🇸🇩 Sudan
            ('🇸🇪', None),                         # 🇸🇪 Sweden
            ('🇸🇬', None),                         # 🇸🇬 Singapore
            ('🇸🇭', None),                         # 🇸🇭 St. Helena
            ('🇸🇮', None),                         # 🇸🇮 Slovenia
            ('🇸🇯', None),                         # 🇸🇯 Svalbard & Jan Mayen
            ('🇸🇰', None),                         # 🇸🇰 Slovakia
            ('🇸🇱', None),                         # 🇸🇱 Sierra Leone
            ('🇸🇲', None),                         # 🇸🇲 San Marino
            ('🇸🇳', None),                         # 🇸🇳 Senegal
            ('🇸🇴', None),                         # 🇸🇴 Somalia
            ('🇸🇷', None),                         # 🇸🇷 Suriname
            ('🇸🇸', None),                         # 🇸🇸 South Sudan
            ('🇸🇹', None),                         # 🇸🇹 São Tomé & Príncipe
            ('🇸🇻', None),                         # 🇸🇻 El Salvador
            ('🇸🇽', None),                         # 🇸🇽 Sint Maarten
            ('🇸🇾', None),                         # 🇸🇾 Syria
            ('🇸🇿', None),                         # 🇸🇿 Swaziland
            ('🇹🇦', None),                         # 🇹🇦 Tristan da Cunha
            ('🇹🇨', None),                         # 🇹🇨 Turks & Caicos Islands
            ('🇹🇩', None),                         # 🇹🇩 Chad
            ('🇹🇫', None),                         # 🇹🇫 French Southern Territories
            ('🇹🇬', None),                         # 🇹🇬 Togo
            ('🇹🇭', None),                         # 🇹🇭 Thailand
            ('🇹🇯', None),                         # 🇹🇯 Tajikistan
            ('🇹🇰', None),                         # 🇹🇰 Tokelau
            ('🇹🇱', None),                         # 🇹🇱 Timor-Leste
            ('🇹🇲', None),                         # 🇹🇲 Turkmenistan
            ('🇹🇳', None),                         # 🇹🇳 Tunisia
            ('🇹🇴', None),                         # 🇹🇴 Tonga
            ('🇹🇷', None),                         # 🇹🇷 Turkey
            ('🇹🇹', None),                         # 🇹🇹 Trinidad & Tobago
            ('🇹🇻', None),                         # 🇹🇻 Tuvalu
            ('🇹🇼', None),                         # 🇹🇼 Taiwan
            ('🇹🇿', None),                         # 🇹🇿 Tanzania
            ('🇺🇦', None),                         # 🇺🇦 Ukraine
            ('🇺🇬', None),                         # 🇺🇬 Uganda
            ('🇺🇲', None),                         # 🇺🇲 U.S. Outlying Islands
            ('🇺🇸', None),                         # 🇺🇸 United States
            ('🇺🇾', None),                         # 🇺🇾 Uruguay
            ('🇺🇿', None),                         # 🇺🇿 Uzbekistan
            ('🇻🇦', None),                         # 🇻🇦 Vatican City
            ('🇻🇨', None),                         # 🇻🇨 St. Vincent & Grenadines
            ('🇻🇪', None),                         # 🇻🇪 Venezuela
            ('🇻🇬', None),                         # 🇻🇬 British Virgin Islands
            ('🇻🇮', None),                         # 🇻🇮 U.S. Virgin Islands
            ('🇻🇳', None),                         # 🇻🇳 Vietnam
            ('🇻🇺', None),                         # 🇻🇺 Vanuatu
            ('🇼🇫', None),                         # 🇼🇫 Wallis & Futuna
            ('🇼🇸', None),                         # 🇼🇸 Samoa
            ('🇽🇰', None),                         # 🇽🇰 Kosovo
            ('🇾🇪', None),                         # 🇾🇪 Yemen
            ('🇾🇹', None),                         # 🇾🇹 Mayotte
            ('🇿🇦', None),                         # 🇿🇦 South Africa
            ('🇿🇲', None),                         # 🇿🇲 Zambia
            ('🇿🇼', None),                         # 🇿🇼 Zimbabwe
        ]],
]



# -*- coding: utf-8 -*-

# Copyright © 2026 Onboard contributors
#
# This file is part of Onboard.
#
# Onboard is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Onboard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""
Wayland-related runtime helpers.

This module isolates all Wayland / GtkLayerShell concerns so the rest of the
code base doesn't have to repeat the same try/except dance.
"""

from __future__ import division, print_function, unicode_literals

import os
import logging

import gi
from Onboard.Version import require_gi_versions
require_gi_versions()
from gi.repository import Gdk, Gio, GLib

_logger = logging.getLogger(__name__)


_layer_shell = None
_layer_shell_checked = False


def is_wayland():
    """
    Return True if Onboard is currently running on a Wayland session.

    Detection is done via the GDK backend of the default display, falling back
    to the ``XDG_SESSION_TYPE`` / ``WAYLAND_DISPLAY`` env vars when GDK has not
    been initialised yet (e.g. during very early startup or unit tests).
    """
    display = Gdk.Display.get_default()
    if display is not None:
        type_name = type(display).__name__
        # Accept both GdkWaylandDisplay and the introspected name variants.
        if "Wayland" in type_name:
            return True
        if "X11" in type_name:
            return False

    # Fallback: env-based detection. Useful before GDK is initialised.
    if os.environ.get("WAYLAND_DISPLAY"):
        return True
    if os.environ.get("XDG_SESSION_TYPE", "").lower() == "wayland":
        return True
    return False


def get_session_type_label():
    """ Short human-readable session label for log lines. """
    if is_wayland():
        desktop = os.environ.get("XDG_CURRENT_DESKTOP", "").split(":")[0] or \
                  os.environ.get("DESKTOP_SESSION", "") or "unknown"
        return "Wayland ({})".format(desktop)
    return "X11"


def is_kde_plasma():
    """
    True if we're running inside a KDE Plasma session. Used to enable the
    KWin-window-rule path (regular toplevel + acceptfocus rule) instead of
    gtk-layer-shell, which preserves the user's ability to drag/resize the
    keyboard window. Other Wayland compositors don't expose an equivalent
    rule mechanism, so they still have to use layer-shell.
    """
    desktop = os.environ.get("XDG_CURRENT_DESKTOP", "")
    if "KDE" in desktop or "Plasma" in desktop:
        return True
    if os.environ.get("KDE_FULL_SESSION") == "true":
        return True
    return False


def install_kwin_rule(app_id="onboard",
                     rule_name="onboard",
                     description="Onboard on-screen keyboard"):
    """
    Idempotently install a KWin window rule that marks the Onboard window
    as not-focus-stealing and always-on-top. Returns True on success.

    The rule is written to ``~/.config/kwinrulesrc`` (user scope, no sudo).
    KWin is asked to reload via D-Bus; if that fails the change still
    takes effect on next KWin reconfigure / next login.

    The function compares the desired configuration against what's already on
    disk and only rewrites the file (and triggers ``KWin reconfigure``) when
    something actually changed. Self-healing -- if the user deletes the rule
    by hand it is recreated -- without spamming sync clients or stomping on
    custom edits between runs.

    This is the same trick `vboard <https://github.com/archisman-panigrahi/vboard>`_
    uses on KDE Wayland to keep its keyboard window draggable while still
    refusing keyboard focus.
    """
    import configparser
    import io
    import subprocess

    cfg_dir = os.environ.get("XDG_CONFIG_HOME") or \
              os.path.expanduser("~/.config")
    cfg_file = os.path.join(cfg_dir, "kwinrulesrc")

    parser = configparser.ConfigParser(interpolation=None,
                                        allow_no_value=True)
    parser.optionxform = str  # preserve case (KWin keys are CamelCase)

    current_text = ""
    if os.path.exists(cfg_file):
        try:
            with open(cfg_file, "r", encoding="utf-8") as f:
                current_text = f.read()
            parser.read_string(current_text)
        except (OSError, configparser.Error) as e:
            _logger.warning("Could not parse %s: %s -- aborting "
                            "KWin rule install", cfg_file, e)
            return False

    if not parser.has_section("General"):
        parser.add_section("General")
    if not parser.has_section(rule_name):
        parser.add_section(rule_name)

    # Drop any stale per-desktop overrides ([$d]) and legacy title matches
    for opt in list(parser.options(rule_name)):
        if opt.endswith("[$d]"):
            parser.remove_option(rule_name, opt)
    for legacy in ("title", "titlematch"):
        parser.remove_option(rule_name, legacy)

    # KWin's rule format stores the Wayland app_id under the historical
    # 'wmclass' keys. 'wmclassmatch=1' = exact match, 'wmclasscomplete=false'
    # = match against the resource class only (single token).
    # 'aboverule=2' / 'acceptfocusrule=2' = "force this value" (vs. "do once",
    # "remember", etc.).
    values = {
        "Description": description,
        "Enabled": "true",
        "wmclass": app_id,
        "wmclassmatch": "1",
        "wmclasscomplete": "false",
        # Only match Normal windows (NormalMask = 1). Without this,
        # the rule also catches Dialog (=32) / Utility (=64) windows of
        # the same app -- and forcing acceptfocus=false on those leaves
        # e.g. the "New snippet" dialog unable to receive keyboard input.
        "types": "1",
        "typesrule": "2",
        "above": "true",
        "aboverule": "2",
        "acceptfocus": "false",
        "acceptfocusrule": "2",
        "skiptaskbar": "true",
        "skiptaskbarrule": "2",
        "skippager": "true",
        "skippagerrule": "2",
    }
    for k, v in values.items():
        parser.set(rule_name, k, v)

    rules = [r.strip()
             for r in parser.get("General", "rules", fallback="").split(",")
             if r.strip()]
    if rule_name not in rules:
        rules.append(rule_name)
    parser.set("General", "rules", ",".join(rules))
    parser.set("General", "count", str(len(rules)))

    # Render the desired state to a string and bail out early if it matches
    # what's already on disk -- avoids touching mtime (sync churn) and saves
    # a needless KWin reconfigure round-trip on every Onboard start.
    buf = io.StringIO()
    parser.write(buf)
    new_text = buf.getvalue()

    if new_text == current_text:
        _logger.debug("KWin window rule '%s' already up to date at %s",
                      rule_name, cfg_file)
        return True

    # Atomic write: tmp file in same dir + rename, so a crash mid-write
    # can't leave kwinrulesrc truncated.
    tmp_file = cfg_file + ".onboard.tmp"
    try:
        os.makedirs(cfg_dir, exist_ok=True)
        with open(tmp_file, "w", encoding="utf-8") as f:
            f.write(new_text)
        os.replace(tmp_file, cfg_file)
    except OSError as e:
        _logger.warning("Could not write %s: %s", cfg_file, e)
        try:
            os.unlink(tmp_file)
        except OSError:
            pass
        return False

    # Hot-reload KWin so the rule takes effect immediately. Only fired when
    # we actually changed the file.
    for cmd in (["qdbus6", "org.kde.KWin", "/KWin", "reconfigure"],
                ["qdbus", "org.kde.KWin", "/KWin", "reconfigure"],
                ["dbus-send", "--session", "--dest=org.kde.KWin",
                 "/KWin", "org.kde.KWin.reconfigure"]):
        try:
            subprocess.run(cmd, check=True, timeout=2,
                           capture_output=True)
            break
        except (FileNotFoundError, subprocess.SubprocessError,
                subprocess.TimeoutExpired):
            continue

    _logger.info("KWin window rule '%s' updated at %s "
                 "(app_id=%s, acceptfocus=false, above=true)",
                 rule_name, cfg_file, app_id)
    return True


def _get_layer_shell():
    """ Lazy-import GtkLayerShell. Returns the module or None. """
    global _layer_shell, _layer_shell_checked
    if _layer_shell_checked:
        return _layer_shell
    _layer_shell_checked = True
    try:
        gi.require_version("GtkLayerShell", "0.1")
        from gi.repository import GtkLayerShell
        _layer_shell = GtkLayerShell
        _logger.debug("GtkLayerShell loaded")
    except (ValueError, ImportError) as e:
        _logger.info("GtkLayerShell not available: {}".format(e))
        _layer_shell = None
    return _layer_shell


def is_layer_shell_supported_by_compositor():
    """
    True only when the running Wayland compositor actually exposes
    ``zwlr_layer_shell_v1``. Distinct from :func:`is_layer_shell_available`,
    which only checks whether the GtkLayerShell typelib is importable.

    Mutter (GNOME) advertises no such global, so this returns False
    there even though the library is installed system-wide on most distros.
    """
    ls = _get_layer_shell()
    if ls is None:
        return False
    if not hasattr(ls, "is_supported"):
        return False
    try:
        return bool(ls.is_supported())
    except Exception:
        return True


def is_layer_shell_available():
    """
    True if we can usefully use gtk-layer-shell on this system -- the
    typelib is importable *and* the running compositor advertises the
    protocol.
    """
    return is_wayland() and is_layer_shell_supported_by_compositor()


def is_gnome_shell():
    """
    True if we are running inside a GNOME Shell session (Mutter,
    either Wayland or Xorg).
    """
    desktop = os.environ.get("XDG_CURRENT_DESKTOP", "")
    if "GNOME" in desktop:
        return True
    if os.environ.get("GNOME_DESKTOP_SESSION_ID"):
        return True
    return False


def init_layer_shell(window, namespace="onboard"):
    """
    Turn ``window`` into a layer-shell surface (idempotent).

    Returns True on success, False if layer-shell is unavailable or if the
    window is already realised (layer-shell must be initialised before
    realisation).
    """
    ls = _get_layer_shell()
    if ls is None:
        return False

    # Guard against the GNOME-Mutter scenario where the typelib is
    # installed but the compositor doesn't expose ``zwlr_layer_shell_v1``.
    if not ls.is_supported():
        _logger.info("init_layer_shell(): compositor does not expose "
                     "zwlr_layer_shell_v1 -- skipping (no layer surface)")
        return False

    if ls.is_layer_window(window):
        return True

    if window.get_realized():
        # Once a Gdk window has been created, gtk-layer-shell can no longer
        # promote it to a layer surface. The caller should arrange for
        # init_layer_shell() to be called before realize().
        _logger.warning("init_layer_shell(): window already realised; "
                        "layer-shell not applied")
        return False

    ls.init_for_window(window)
    ls.set_namespace(window, namespace)

    # Sit above normal application windows but below regular OVERLAY surfaces
    # like notifications. TOP is what every other on-screen keyboard
    # (squeekboard, maliit-keyboard, wvkbd) uses.
    ls.set_layer(window, ls.Layer.TOP)

    # Never take keyboard focus -- this is the Wayland equivalent of the X
    # ``set_accept_focus(False)`` call that the CHANGELOG flagged as the
    # original Wayland blocker.
    ls.set_keyboard_mode(window, ls.KeyboardMode.NONE)

    _logger.debug("init_layer_shell(): namespace=%s layer=TOP "
                  "keyboard=NONE", namespace)
    return True


def set_anchor_bottom(window, expand=False):
    """
    Anchor a layer-shell window to the bottom edge.
    If ``expand`` is True, also anchor left+right so the surface spans the
    full output width (matches the X "expanded dock" behavior).
    """
    ls = _get_layer_shell()
    if ls is None or not ls.is_layer_window(window):
        return
    ls.set_anchor(window, ls.Edge.BOTTOM, True)
    ls.set_anchor(window, ls.Edge.TOP, False)
    ls.set_anchor(window, ls.Edge.LEFT, expand)
    ls.set_anchor(window, ls.Edge.RIGHT, expand)


def set_anchor_top(window, expand=False):
    """ Anchor a layer-shell window to the top edge. """
    ls = _get_layer_shell()
    if ls is None or not ls.is_layer_window(window):
        return
    ls.set_anchor(window, ls.Edge.TOP, True)
    ls.set_anchor(window, ls.Edge.BOTTOM, False)
    ls.set_anchor(window, ls.Edge.LEFT, expand)
    ls.set_anchor(window, ls.Edge.RIGHT, expand)


def clear_anchors(window):
    """ Detach the window from screen edges (i.e. floating mode). """
    ls = _get_layer_shell()
    if ls is None or not ls.is_layer_window(window):
        return
    for edge in (ls.Edge.TOP, ls.Edge.BOTTOM, ls.Edge.LEFT, ls.Edge.RIGHT):
        ls.set_anchor(window, edge, False)


def set_exclusive_zone(window, height):
    """
    Replacement for ``_NET_WM_STRUT_PARTIAL`` on Wayland.
    ``height`` -- pixels of vertical screen space the keyboard reserves
    (i.e. workareas of maximized apps shrink by this much).

    Pass 0 to clear / -1 to opt out and let other surfaces overlap us.
    """
    ls = _get_layer_shell()
    if ls is None or not ls.is_layer_window(window):
        return
    ls.set_exclusive_zone(window, int(height))


def is_layer_window(window):
    """ True if ``window`` was successfully turned into a layer surface. """
    ls = _get_layer_shell()
    if ls is None:
        return False
    return ls.is_layer_window(window)


def set_layer_keyboard_interactive(window, interactive):
    """
    Toggle a layer-shell window between ON_DEMAND and NONE keyboard
    modes. No-op on non-layer windows or when gtk-layer-shell is
    unavailable.

    The layer surface is created with KeyboardMode.NONE so it never
    steals focus from the app being typed into. Promote to ON_DEMAND
    when a focusable child (snippet dialog, ...) needs to receive
    input, and drop back to NONE when it closes.
    """
    ls = _get_layer_shell()
    if ls is None or not ls.is_layer_window(window):
        return
    mode = ls.KeyboardMode.ON_DEMAND if interactive else ls.KeyboardMode.NONE
    ls.set_keyboard_mode(window, mode)


def diagnose_uinput_event_device(device_name="Onboard on-screen keyboard",
                                 wait_seconds=0.5):
    """
    After ``select_backend(BACKEND_UINPUT)`` has succeeded, verify that the
    resulting ``/dev/input/eventN`` is actually readable by the current user.
    Returns ``(ok, message)``.

    On Wayland the kernel-level uinput open can succeed (so Onboard *thinks*
    everything is fine) while the synthesized events still go nowhere because
    the new event device is owned ``root:input 0660`` and the compositor,
    running as the user, can't read from it. This function detects exactly
    that failure mode and returns an actionable error string.
    """
    import os
    import glob
    import time

    # Give udev a moment to finish processing the new device.
    time.sleep(wait_seconds)

    candidate = None
    newest_mtime = 0
    for path in glob.glob("/dev/input/event*"):
        try:
            with open("/sys/class/input/{}/device/name"
                      .format(os.path.basename(path))) as f:
                if f.read().strip() == device_name:
                    st = os.stat(path)
                    if st.st_mtime > newest_mtime:
                        newest_mtime = st.st_mtime
                        candidate = path
        except (OSError, IOError):
            continue

    if candidate is None:
        return (False,
                "uinput device '{}' not found under /dev/input/event*; "
                "udev may not have processed the new device yet"
                .format(device_name))

    if not os.access(candidate, os.R_OK):
        return (False,
                "Found {} but it is not readable by the current user. "
                "The Wayland compositor needs read access "
                "to receive injected keystrokes. Install the udev rule to "
                "/etc/udev/rules.d/72-onboard-uinput.rules and run "
                "'sudo udevadm control --reload-rules && "
                "sudo udevadm trigger /dev/uinput', then restart Onboard."
                .format(candidate))

    # Note: there's no runtime check for the 'power-switch' udev tag here
    # any more. systemd-logind only EVIOCGRABs power-switch-tagged devices
    # that actually advertise KEY_POWER/KEY_SLEEP/etc. — we strip those
    # capabilities at uinput-creation time in osk_uinput.c, so the tag is
    # harmless on us. The previous TAG-=power-switch in our rule was purely
    # defensive and was dropped along with the runtime check.

    return (True, "uinput event device {} is readable".format(candidate))


# ---------------------------------------------------------------------------
# KDE layout-change watcher
# ---------------------------------------------------------------------------
#
# On Wayland the keymap-state signals Onboard listens to on X11 are silent
# for windows that refuse keyboard focus, so when the user switches layout
# via the KDE taskbar/tray the keyboard window never learns about it
# and the key labels stay on the previous layout until e.g. the next pointer enter
# happens to repaint the surface.
#
# KDE Plasma has a focus-independent session-bus signal for this
# exact use case. The contract is identical on X11 and Wayland,
# on X11 it is hosted by the kded "keyboard" module, on Wayland by KWin itself.
#
# Quirk: KWin only registers ``/Layouts`` when at least two layouts are
# configured -- before that the bus name has no owner, and after the user
# adds a second layout the name appears. We therefore also watch
# NameOwnerChanged so we can attach late if the interface shows up
# mid-session.

_KDE_KBD_BUS_NAME  = "org.kde.keyboard"
_KDE_KBD_OBJECT    = "/Layouts"
_KDE_KBD_INTERFACE = "org.kde.KeyboardLayouts"


class KdeLayoutWatcher:
    """
    Subscribes to KDE's ``org.kde.KeyboardLayouts.layoutChanged`` signal
    so Onboard learns about layout switches even when its window has no
    keyboard focus.

    The ``callback`` is invoked as ``callback(new_index)`` on every
    ``layoutChanged`` *or* ``layoutListChanged`` emission and on initial
    attach. ``new_index`` is the *authoritative* layout index reported
    by KDE (queried via ``getLayout()`` so we get the same value
    regardless of whether the signal carried it). It is ``-1`` only if
    the D-Bus query unexpectedly fails -- callers should treat that as
    "trigger a refresh but don't sync the cached xkb_state".

    The index lets the caller bypass our local wl_keyboard-fed
    ``xkb_state``, which is stale because Onboard's surfaces don't take
    keyboard focus -- subscribers can push the new index into
    ``vk.lock_group(index)`` so subsequent ``labels_from_keycode()``
    lookups query the right layout.

    Callers are still expected to debounce expensive work;
    ``OnboardGtk.cb_group_changed`` does so via the ``_reload_layout_timer``.

    Construction is safe even when KDE Plasma is not running or the
    ``/Layouts`` interface has not been registered yet (single-layout
    configurations); all D-Bus errors are caught and logged, never
    raised. A NameOwnerChanged watch handles late binding when the user
    adds a second layout at runtime.

    Call ``stop()`` to release subscriptions before quitting.
    """

    def __init__(self, callback):
        self._callback = callback
        self._bus = None
        self._proxy = None
        self._signal_handler_id = 0
        self._name_owner_sub_id = 0

        try:
            self._bus = Gio.bus_get_sync(Gio.BusType.SESSION, None)
        except GLib.Error as e:
            _logger.warning("KdeLayoutWatcher: cannot acquire session "
                            "bus: %s", e.message)
            return

        # Late-binding watch first, so that if /Layouts isn't registered
        # yet (single-layout config) and the user adds a second layout
        # later we still get the binding.
        try:
            self._name_owner_sub_id = self._bus.signal_subscribe(
                "org.freedesktop.DBus",        # sender
                "org.freedesktop.DBus",        # interface
                "NameOwnerChanged",            # member
                "/org/freedesktop/DBus",       # path
                _KDE_KBD_BUS_NAME,             # arg0 filter
                Gio.DBusSignalFlags.NONE,
                self._on_name_owner_changed,
            )
        except GLib.Error as e:
            _logger.warning("KdeLayoutWatcher: NameOwnerChanged "
                            "subscribe failed: %s", e.message)

        self._try_attach_proxy()

    # ------------------------------------------------------------------
    # public API
    # ------------------------------------------------------------------

    def stop(self):
        """ Drop the proxy + NameOwnerChanged subscription. """
        self._detach_proxy()
        if self._bus is not None and self._name_owner_sub_id:
            try:
                self._bus.signal_unsubscribe(self._name_owner_sub_id)
            except GLib.Error:
                pass
            self._name_owner_sub_id = 0
        self._bus = None

    # ------------------------------------------------------------------
    # internals
    # ------------------------------------------------------------------

    def _try_attach_proxy(self):
        """
        Create the proxy and check whether the bus name actually has an
        owner. ``Gio.DBusProxy.new_sync`` with the default flags happily
        returns a proxy for a non-existent name; we treat
        ``name_owner == None`` as "interface not registered yet" and
        wait for NameOwnerChanged.
        """
        if self._proxy is not None:
            return                                     # already attached

        try:
            proxy = Gio.DBusProxy.new_sync(
                self._bus,
                Gio.DBusProxyFlags.DO_NOT_AUTO_START,
                None,                                  # interface info
                _KDE_KBD_BUS_NAME,
                _KDE_KBD_OBJECT,
                _KDE_KBD_INTERFACE,
                None,                                  # cancellable
            )
        except GLib.Error as e:
            _logger.debug("KdeLayoutWatcher: proxy creation failed: %s",
                          e.message)
            return

        if proxy.get_name_owner() is None:
            _logger.info("KdeLayoutWatcher: %s not yet registered "
                         "(single-layout config?); waiting for it to "
                         "appear", _KDE_KBD_BUS_NAME)
            return

        self._proxy = proxy
        self._signal_handler_id = proxy.connect(
            "g-signal", self._on_g_signal)
        _logger.info("KdeLayoutWatcher: attached to %s %s %s",
                     _KDE_KBD_BUS_NAME, _KDE_KBD_OBJECT,
                     _KDE_KBD_INTERFACE)
        # Initial sync: the user may have switched layout before Onboard
        # finished starting; without this they'd see the wrong labels
        # until the next switch.
        self._fire("initial attach")

    def _detach_proxy(self):
        if self._proxy is not None and self._signal_handler_id:
            try:
                self._proxy.disconnect(self._signal_handler_id)
            except Exception:
                pass
        self._signal_handler_id = 0
        self._proxy = None

    def _query_current_index(self):
        """
        Ask KDE which layout is currently active. Returns the int index
        or ``-1`` on error. We always query rather than trusting the
        signal payload because ``layoutListChanged`` carries no index
        and the initial attach has none either.
        """
        if self._proxy is None:
            return -1
        try:
            result = self._proxy.call_sync(
                "getLayout",
                None,                          # parameters
                Gio.DBusCallFlags.NO_AUTO_START,
                500,                           # timeout ms
                None,                          # cancellable
            )
        except GLib.Error as e:
            _logger.warning("KdeLayoutWatcher: getLayout() failed: %s",
                            e.message)
            return -1
        try:
            (idx,) = result.unpack()
            return int(idx)
        except Exception:
            _logger.warning("KdeLayoutWatcher: unexpected getLayout() "
                            "reply shape: %s", result)
            return -1

    def _fire(self, reason):
        idx = self._query_current_index()
        _logger.debug("KdeLayoutWatcher: %s -> index %d", reason, idx)
        try:
            self._callback(idx)
        except Exception:
            _logger.exception("KdeLayoutWatcher: callback raised")

    def _on_g_signal(self, _proxy, _sender, signal_name, _parameters):
        # Both signals warrant a refresh: 'layoutChanged' is the normal
        # switch event, 'layoutListChanged' fires when the user edits
        # the set of configured layouts -- either case means our cached
        # group <-> language mapping is stale.
        if signal_name in ("layoutChanged", "layoutListChanged"):
            self._fire(signal_name)

    def _on_name_owner_changed(self, _bus, _sender, _object,
                               _interface, _signal, parameters):
        try:
            name, _old_owner, new_owner = parameters.unpack()
        except Exception:
            return
        if name != _KDE_KBD_BUS_NAME:
            return
        if new_owner:
            _logger.debug("KdeLayoutWatcher: %s appeared (owner=%s); "
                          "attaching", name, new_owner)
            self._try_attach_proxy()
            # Also trigger an immediate sync -- between us missing the
            # prior layoutChanged events and now, the active group may
            # already differ from what Onboard has cached.
            if self._proxy is not None:
                self._fire("NameOwnerChanged (acquired)")
        else:
            _logger.debug("KdeLayoutWatcher: %s went away", name)
            self._detach_proxy()
